# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
import os

def cargar(ruta):
    """Intenta cargar la imagen indicada por el argumento ``ruta``.

    Por ejemplo::

        import pilas

        imagen = pilas.imagenes.cargar("mi_archivo.png")

    En caso de éxito retorna el objeto Image, que se puede asignar
    a un Actor.

    El directorio de búsqueda de la imagen sigue el siguiente orden:

        * primero busca en el directorio actual.
        * luego en 'data'.
        * por último en el directorio estándar de la biblioteca.

    En caso de error genera una excepción de tipo IOError.
    """

    if not pilas.mundo:
        mensaje = "Tiene que invocar a la funcion ``pilas.iniciar()`` para comenzar."
        print mensaje
        raise Exception(mensaje)
    
    ruta = pilas.utils.obtener_ruta_al_recurso(ruta)
    return pilas.mundo.motor.cargar_imagen(ruta)

def cargar_grilla(ruta, columnas=1, filas=1):
    """Representa una grilla de imagenes con varios cuadros de animación.

    Una grilla es un objeto que se tiene que inicializar con la ruta
    a una imagen, la cantidad de columnas y filas.

    Por ejemplo, si tenemos una grilla con 2 columnas y 3 filas
    podemos asociarla a un actor de la siguiente manera::

        grilla = pilas.imagenes.cargar_grilla("animacion.png", 2, 3)
        grilla.asignar(actor)

    Entonces, a partir de ahora nuestro actor muestra solamente un
    cuadro de toda la grilla.

    Si quieres avanzar la animacion tienes que modificar el objeto
    grilla y asignarlo nuevamente al actor::

        grilla.avanzar()
        grilla.asignar(actor)
    """
    if not pilas.mundo:
        mensaje = "Tiene que invocar a la funcion ``pilas.iniciar()`` para comenzar."
        print mensaje
        raise Exception(mensaje)
    
    ruta = pilas.utils.obtener_ruta_al_recurso(ruta)
    return pilas.mundo.motor.obtener_grilla(ruta, columnas, filas)

def cargar_lienzo():
    """Representa un rectangulo (inicialmente transparente) para dibujar."""
    return pilas.mundo.motor.obtener_lienzo()

def cargar_superficie(ancho, alto):
    return pilas.mundo.motor.obtener_superficie(ancho, alto)
    
cargar_imagen = cargar



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui.ui'
#
# Created: Sun Aug 21 13:19:30 2011
#      by: PyQt4 UI code generator 4.8.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName(_fromUtf8("MainWindow"))
        MainWindow.resize(822, 602)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
        self.gridLayout = QtGui.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.splitter_2 = QtGui.QSplitter(self.centralwidget)
        self.splitter_2.setOrientation(QtCore.Qt.Horizontal)
        self.splitter_2.setObjectName(_fromUtf8("splitter_2"))
        self.vbox_left = QtGui.QWidget(self.splitter_2)
        self.vbox_left.setObjectName(_fromUtf8("vbox_left"))
        self.vlayout_left = QtGui.QVBoxLayout(self.vbox_left)
        self.vlayout_left.setMargin(0)
        self.vlayout_left.setObjectName(_fromUtf8("vlayout_left"))
        self.arbol = QtGui.QTreeWidget(self.vbox_left)
        self.arbol.setObjectName(_fromUtf8("arbol"))
        self.arbol.headerItem().setText(0, _fromUtf8("1"))
        self.vlayout_left.addWidget(self.arbol)
        self.splitter = QtGui.QSplitter(self.splitter_2)
        self.splitter.setOrientation(QtCore.Qt.Vertical)
        self.splitter.setObjectName(_fromUtf8("splitter"))
        self.imagen = QtGui.QGraphicsView(self.splitter)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.imagen.sizePolicy().hasHeightForWidth())
        self.imagen.setSizePolicy(sizePolicy)
        self.imagen.setMinimumSize(QtCore.QSize(400, 300))
        self.imagen.setAutoFillBackground(True)
        self.imagen.setRenderHints(QtGui.QPainter.SmoothPixmapTransform)
        self.imagen.setTransformationAnchor(QtGui.QGraphicsView.AnchorUnderMouse)
        self.imagen.setResizeAnchor(QtGui.QGraphicsView.AnchorViewCenter)
        self.imagen.setViewportUpdateMode(QtGui.QGraphicsView.SmartViewportUpdate)
        self.imagen.setObjectName(_fromUtf8("imagen"))
        self.layoutWidget = QtGui.QWidget(self.splitter)
        self.layoutWidget.setObjectName(_fromUtf8("layoutWidget"))
        self.verticalLayout = QtGui.QVBoxLayout(self.layoutWidget)
        self.verticalLayout.setMargin(0)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.progreso = QtGui.QFrame(self.layoutWidget)
        self.progreso.setMinimumSize(QtCore.QSize(0, 22))
        self.progreso.setFrameShape(QtGui.QFrame.NoFrame)
        self.progreso.setFrameShadow(QtGui.QFrame.Plain)
        self.progreso.setLineWidth(0)
        self.progreso.setObjectName(_fromUtf8("progreso"))
        self.label = QtGui.QLabel(self.progreso)
        self.label.setGeometry(QtCore.QRect(6, 0, 170, 21))
        self.label.setObjectName(_fromUtf8("label"))
        self.verticalLayout.addWidget(self.progreso)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.ejecutar = QtGui.QPushButton(self.layoutWidget)
        self.ejecutar.setObjectName(_fromUtf8("ejecutar"))
        self.horizontalLayout.addWidget(self.ejecutar)
        self.fuente = QtGui.QPushButton(self.layoutWidget)
        self.fuente.setObjectName(_fromUtf8("fuente"))
        self.horizontalLayout.addWidget(self.fuente)
        self.guardar = QtGui.QPushButton(self.layoutWidget)
        self.guardar.setObjectName(_fromUtf8("guardar"))
        self.horizontalLayout.addWidget(self.guardar)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.codigo = QtGui.QTextEdit(self.layoutWidget)
        self.codigo.setReadOnly(True)
        self.codigo.setObjectName(_fromUtf8("codigo"))
        self.verticalLayout.addWidget(self.codigo)
        self.gridLayout.addWidget(self.splitter_2, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 822, 23))
        self.menubar.setObjectName(_fromUtf8("menubar"))
        self.menuArchivo = QtGui.QMenu(self.menubar)
        self.menuArchivo.setObjectName(_fromUtf8("menuArchivo"))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName(_fromUtf8("statusbar"))
        MainWindow.setStatusBar(self.statusbar)
        self.actionSalir = QtGui.QAction(MainWindow)
        self.actionSalir.setObjectName(_fromUtf8("actionSalir"))
        self.menuArchivo.addAction(self.actionSalir)
        self.menubar.addAction(self.menuArchivo.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "Ejemplos del motor pilas", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("MainWindow", "Aguarde por favor ...", None, QtGui.QApplication.UnicodeUTF8))
        self.ejecutar.setText(QtGui.QApplication.translate("MainWindow", "Ejecutar", None, QtGui.QApplication.UnicodeUTF8))
        self.fuente.setText(QtGui.QApplication.translate("MainWindow", "Cambiar tipografia", None, QtGui.QApplication.UnicodeUTF8))
        self.guardar.setText(QtGui.QApplication.translate("MainWindow", "Guardar Codigo", None, QtGui.QApplication.UnicodeUTF8))
        self.menuArchivo.setTitle(QtGui.QApplication.translate("MainWindow", "Archivo", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSalir.setText(QtGui.QApplication.translate("MainWindow", "Salir", None, QtGui.QApplication.UnicodeUTF8))




from xml.dom import minidom

class XmlNode:
    """An XML node represents a single field in an XML document."""

    def __init__(self, domElement):
        """Construct an XML node from a DOM element."""
        self.elem = domElement
    
    def getData(self):
        """Extract data from a DOM node."""
        for child in self.elem.childNodes:
            if child.nodeType == child.TEXT_NODE:
                return str(child.data)
        return None

    def getAttributeValue(self, name):
        """Returns the value of the attribute having the specified name."""
        return str(self.elem.attributes[name].value)

    def getChild(self, tag):
        """Returns the first child node having the specified tag."""
        return XmlNode(self.elem.getElementsByTagName(tag)[0])
    
    def getChildren(self, tag):
        """Returns a list of child nodes having the specified tag."""
        return [XmlNode(x) for x in self.elem.getElementsByTagName(tag)]


def makeRootNode(xmlFileName):
    """Creates the root node from an XML file."""
    return XmlNode(minidom.parse(xmlFileName))




# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Actor
from pilas.comportamientos import Comportamiento

VELOCIDAD = 100


class Martian(Actor):

    def __init__(self, x=0, y=0):
        Actor.__init__(self, x=x, y=y)
        self.imagen = pilas.imagenes.cargar_grilla("marcianitos/martian.png", 12)
        self.definir_cuadro(0)
        self.figura = pilas.fisica.Circulo(0, 0, 18, friccion=0, amortiguacion=0, restitucion=0.1)
        self.hacer(Esperando())

    def definir_cuadro(self, indice):
        self.imagen.definir_cuadro(indice)
        self.definir_centro((32, 123))

    def actualizar(self):
        "Sigue el movimiento de la figura."
        self.x = self.figura.x
        self.y = self.figura.y - 15
        self.figura.rotacion = 0

    def crear_disparo(self):
        if self.espejado:
            rotacion = -90
        else:
            rotacion = 90

        disparo = pilas.actores.Disparo(x=self.x, y=self.y+20, rotacion=rotacion, velocidad=10)

    def puede_saltar(self):
        dx, dy = self.figura.obtener_velocidad_lineal()
        return -2 < dy < 2

class Esperando(Comportamiento):
    "Un actor en posicion normal o esperando a que el usuario pulse alguna tecla."

    def iniciar(self, receptor):
        self.receptor = receptor
        self.receptor.definir_cuadro(0)

    def actualizar(self):

        if pilas.mundo.control.izquierda:
            self.receptor.hacer(Caminando())
        elif pilas.mundo.control.derecha:
            self.receptor.hacer(Caminando())

        if pilas.mundo.control.arriba and self.receptor.puede_saltar():
            self.receptor.hacer(Saltando())

        if pilas.mundo.control.boton:
            self.receptor.hacer(Disparar(self.receptor))

class Caminando(Comportamiento):

    def __init__(self):
        self.cuadros = [1, 1, 1, 2, 2, 2]
        self.paso = 0

    def iniciar(self, receptor):
        self.receptor = receptor

    def actualizar(self):
        self.avanzar_animacion()

        if pilas.mundo.control.izquierda:
            self.receptor.figura.definir_velocidad_lineal(-VELOCIDAD)
            self.receptor.espejado = True
        elif pilas.mundo.control.derecha:
            self.receptor.figura.definir_velocidad_lineal(+VELOCIDAD)
            self.receptor.espejado = False
        else:
            self.receptor.figura.definir_velocidad_lineal(0)
            self.receptor.hacer(Esperando())

        if pilas.mundo.control.arriba:
            self.receptor.hacer(Saltando())

    def avanzar_animacion(self):
        self.paso += 1

        if self.paso >= len(self.cuadros):
            self.paso = 0

        self.receptor.definir_cuadro(self.cuadros[self.paso])

class Saltando(Comportamiento):

    def iniciar(self, receptor):
        self.receptor = receptor
        self.receptor.definir_cuadro(3)
        self.esta_bajando = False
        self.receptor.figura.definir_velocidad_lineal(None, 300)


    def actualizar(self):

        # obtiene la velocidad del personaje para detectar cuando
        # toca el suelo.
        vx, vy = self.receptor.figura.obtener_velocidad_lineal()

        if vy < 0:
            self.esta_bajando = True

        if self.esta_bajando and -2 < vy < 2:
            self.receptor.figura.definir_velocidad_lineal(0,0)
            self.receptor.hacer(Esperando())

        if pilas.mundo.control.izquierda:
            self.receptor.espejado = True
            self.receptor.figura.definir_velocidad_lineal(-VELOCIDAD)
        elif pilas.mundo.control.derecha:
            self.receptor.espejado = False
            self.receptor.figura.definir_velocidad_lineal(VELOCIDAD)
        else:
            self.receptor.figura.definir_velocidad_lineal(0)

class Disparar(Comportamiento):

    def __init__(self, receptor):
        self.cuadros = [6, 6, 7, 7, 8, 8]
        self.paso = 0
        receptor.crear_disparo()

    def actualizar(self):
        termina = self.avanzar_animacion()

        if termina:
            self.receptor.hacer(Esperando())

    def avanzar_animacion(self):
        self.paso += 1

        if self.paso >= len(self.cuadros):
            self.paso = 0
            return True

        self.receptor.definir_cuadro(self.cuadros[self.paso])



import pilas

pilas.iniciar()
pilas.fondos.Pasto()

def cuando_selecciona(opcion_seleccionada):
    pilas.avisar("Ha seleccionado la opcion: " + opcion_seleccionada)

opciones = pilas.interfaz.ListaSeleccion(['hola', 'opcion', 'tres'], cuando_selecciona)
pilas.avisar("Selecciona alguna de las opciones...")

pilas.ejecutar()



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

mundo = None
bg = None

import sys
import utils
from mundo import Mundo
import actores
import fondos
import habilidades
import eventos
import sonidos
import colores
import atajos
import escenas
import ejemplos
import interfaz

__doc__ = """
Módulo pilas
============

Pilas es una biblioteca para facilitar el desarrollo
de videojuegos. Es útil para programadores
principiantes o para el desarrollo de juegos casuales.

Este módulo contiene las funciones principales
para iniciar y ejecutar la biblioteca.
"""

if utils.esta_en_sesion_interactiva():
    utils.cargar_autocompletado()

def iniciar(ancho=640, alto=480, titulo='Pilas', usar_motor='qtgl', 
            rendimiento=60, modo='detectar', economico=True, 
            gravedad=(0, -90), pantalla_completa=False):
    """
    Inicia la ventana principal del juego con algunos detalles de funcionamiento.

    Ejemplo de invocación:

        >>> pilas.iniciar(ancho=320, alto=240)

    .. image:: images/iniciar_320_240.png

    Parámetros:

    :ancho: el tamaño en pixels para la ventana.
    :alto: el tamaño en pixels para la ventana.
    :titulo: el titulo a mostrar en la ventana.
    :usar_motor: el motor multimedia a utilizar, puede ser 'qt' o 'qtgl'.
    :rendimiento: cantidad de cuadros por segundo a mostrar.
    :modo: si se utiliza modo interactivo o no.
    :economico: si tiene que evitar consumir muchos recursos de procesador
    :gravedad: el vector de aceleracion para la simulacion de fisica.
    :pantalla_completa: si debe usar pantalla completa o no.

    """
    
    global mundo

    motor = __crear_motor(usar_motor)
    mundo = Mundo(motor, ancho, alto, titulo, rendimiento, economico, gravedad, pantalla_completa)
    escenas.Normal(colores.grisclaro)


def ejecutar(ignorar_errores=False):
    """Pone en funcionamiento las actualizaciones y dibujado.
    
    Esta función es necesaria cuando se crea un juego
    en modo ``no-interactivo``."""
    mundo.ejecutar_bucle_principal(ignorar_errores)

def terminar():
    """Finaliza la ejecución de pilas y cierra la ventana principal."""
    mundo.terminar()

def ver(objeto, imprimir=True, retornar=False):
    """Imprime en pantalla el codigo fuente asociado a un objeto o elemento de pilas."""
    import inspect

    try:
        codigo = inspect.getsource(objeto.__class__)
    except TypeError:
        codigo = inspect.getsource(objeto)

    if imprimir:
        print codigo

    if retornar:
        return codigo

def version():
    """Retorna el número de version de pilas."""
    import pilasversion

    return pilasversion.VERSION

def __crear_motor(usar_motor):
    """Genera instancia del motor multimedia en base a un nombre.
    
    Esta es una función interna y no debe ser ejecutada
    excepto por el mismo motor pilas."""

    if usar_motor == 'qt':
        from motores import motor_qt
        motor = motor_qt.Qt()
    elif usar_motor == 'qtgl':
        from motores import motor_qt
        motor = motor_qt.QtGL()
    else:
        print "El motor multimedia seleccionado (%s) no esta disponible" %(usar_motor)
        print "Las opciones de motores que puedes probar son 'qt' y 'qtgl'."
        sys.exit(1)

    return motor

def reiniciar():
    """Elimina todos los actores y vuelve al estado inicial."""
    actores.utils.eliminar_a_todos()
    mundo.reiniciar()

anterior_texto = None

def avisar(mensaje):
    """Emite un mensaje en la ventana principal.

    Este mensaje aparecerá en la parte inferior de la pantalla, por
    ejemplo:

        >>> pilas.avisar("Use la tecla <esc> para terminar el programa")
    """
    global anterior_texto
    izquierda, derecha, arriba, abajo = utils.obtener_bordes()

    if anterior_texto:
        anterior_texto.eliminar()

    texto = actores.Texto(mensaje)
    texto.magnitud = 17
    texto.centro = ("centro", "centro")
    texto.izquierda = izquierda + 10
    texto.color = colores.blanco
    texto.abajo = abajo + 10
    anterior_texto = texto

def abrir_cargador():
    """Abre un cargador de ejemplos con varios códigos de prueba.

    Ejemplo:

        >>> pilas.abrir_cargador()

    El cargador de ejemplos se ve de esta forma:

    .. image:: images/cargador.png
    """
    import cargador
    cargador.ejecutar()
    return []



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import ui
import os
import sys
import glob

from PyQt4 import QtGui, QtCore

import syntax
import buscador


MENSAJE_PRESENTACION = u"""Bienvenido al cargador de ejemplos.

Selecciona un ejemplo usando el panel de
la izquierda y luego verás el código acá.
"""


class VentanaPrincipal(QtGui.QMainWindow, ui.Ui_MainWindow):

    def __init__(self):
        self._iniciar_interfaz()
        self.this_dir = os.path.abspath(os.path.dirname(__file__))
        self.example_dir = os.path.join(self.this_dir, 'ejemplos')
        self._cargar_lista_de_ejemplos()
        # LLamar cargar buscador siempre despues de cargar ejemplos
        self._cargar_buscador()

        # Senales
        self.connect(self.ui.ejecutar, QtCore.SIGNAL("clicked()"),
            self.cuando_pulsa_boton_ejecutar)
        self.connect(self.ui.fuente, QtCore.SIGNAL("clicked()"),
            self.cuando_pulsa_boton_fuente)
        self.connect(self.ui.guardar, QtCore.SIGNAL("clicked()"),
            self.cuando_pulsa_boton_guardar)
        self.connect(self.ui.arbol, QtCore.SIGNAL("itemSelectionChanged()"),
            self.cuando_cambia_seleccion)
        self.connect(self.ui.arbol,
            QtCore.SIGNAL("itemActivated(QListWidgetItem *)"),
            self.cuando_pulsa_boton_ejecutar)
        self.connect(self.ui.actionSalir, QtCore.SIGNAL("activated()"),
            self.cuando_quiere_cerrar)

        syntax.PythonHighlighter(self.ui.codigo.document())

        self._definir_estado_habilitado(True)
        self._mostrar_image_inicial()
        self._mostrar_codigo_presentacion_inicial()
        self.centrar_ventana()

    def centrar_ventana(self):
        ancho = self.size().width()
        alto = self.size().height()
        escritorio = QtGui.QDesktopWidget().screenGeometry()
        self.setGeometry(
                    (escritorio.width() - ancho) / 2,
                    (escritorio.height() - alto) / 2, ancho, alto)

    def _definir_estado_habilitado(self, esta_habilitado):
        "Oculta la barra de progreso y habilita todos los controles."

        widgets = [self.locate_widget, self.ui.progreso, self.ui.ejecutar,
            self.ui.guardar, self.ui.arbol, self.ui.fuente, self.ui.codigo,
            self.ui.imagen]

        self.ui.progreso.setVisible(not esta_habilitado)

        for x in widgets:
            x.setEnabled(esta_habilitado)

    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_Return and \
        self.ui.arbol.hasFocus() and \
        self.ui.arbol.currentItem().childCount() == 0:
            self.cuando_pulsa_boton_ejecutar()

        return QtGui.QMainWindow.keyPressEvent(self, event)

    def _cargar_buscador(self):
        self.locate_widget = buscador.ExampleLocatorWidget(
            self.ejemplos, self.ui.arbol)
        self.ui.vlayout_left.addWidget(self.locate_widget)
        shortcut = QtGui.QShortcut(QtGui.QKeySequence(
            QtCore.Qt.CTRL + QtCore.Qt.Key_F), self)
        self.connect(self.locate_widget, QtCore.SIGNAL("itemFound(QString)"),
            self.buscador_resultado_encontrado)
        self.connect(shortcut, QtCore.SIGNAL("activated()"),
            self.locate_widget.setFocus)

    def _cargar_lista_de_ejemplos(self):
        self.ejemplos = {}
        self.ui.arbol.setColumnCount(1)
        self.ui.arbol.setHeaderLabels(["Nombre"])

        directorios = glob.glob(self.example_dir + '/*')

        for directorio in directorios:
            raiz = QtGui.QTreeWidgetItem([os.path.basename(directorio), ""])
            self.ui.arbol.addTopLevelItem(raiz)

            archivos = glob.glob(directorio + '/*.py')

            for archivo in archivos:
                nombre_legible = os.path.basename(archivo).replace(".py", "")
                item = QtGui.QTreeWidgetItem([nombre_legible, archivo])
                raiz.addChild(item)
                self.ejemplos[nombre_legible] = archivo

    def _iniciar_interfaz(self):
        QtGui.QMainWindow.__init__(self)
        ui.Ui_MainWindow.__init__(self)
        self.ui = ui.Ui_MainWindow()
        self.ui.setupUi(self)

    def cuando_pulsa_boton_ejecutar(self):
        nombre_ejemplo = self._obtener_item_actual()
        self._ejecutar_ejemplo(nombre_ejemplo)

    def cuando_quiere_cerrar(self):
        sys.exit(0)

    def cuando_pulsa_boton_fuente(self):
        font = self.ui.codigo.font()
        font, ok = QtGui.QFontDialog.getFont(font)

        if ok:
            self.ui.codigo.setFont(font)

    def cuando_pulsa_boton_guardar(self):
        nombre = self._obtener_item_actual()
        path = unicode(QtGui.QFileDialog.getSaveFileName(self,
                    'Guardar ejemplo',
                    nombre,
                    "py (*.py)"))
        if path:
            contenido = self._obtener_codigo_del_ejemplo(nombre)

            archivo = open(path, "wt")
            archivo.write(contenido)
            archivo.close()

    def cuando_cambia_seleccion(self):
        ruta = self._obtener_item_actual()

        if ruta:
            self._mostrar_codigo_del_ejemplo(ruta)
            self._mostrar_imagen_del_ejemplo(ruta)

    def buscador_resultado_encontrado(self, ruta):
        if ruta:
            item = self._obtener_item_desde_ruta(ruta)

            if item:
                self.ui.arbol.setCurrentItem(item)

    def _obtener_item_desde_ruta(self, ruta):
        nombres = ruta.split('/')
        categoria = nombres[-2]
        nombre = nombres[-1].replace('.py', '')

        items = self.ui.arbol.findItems(nombre, QtCore.Qt.MatchRecursive)

        if items:
            return items[0]

    def _mostrar_imagen_del_ejemplo(self, ruta):
        escena = QtGui.QGraphicsScene()
        self.ui.imagen.setScene(escena)
        pixmap = QtGui.QGraphicsPixmapItem(QtGui.QPixmap(
            ruta.replace('.py', '.png')))
        escena.addItem(pixmap)

    def _mostrar_image_inicial(self):
        path = os.path.join(os.path.dirname(__file__), "data/_presentacion")
        self._mostrar_imagen_del_ejemplo(path)

    def _mostrar_codigo_del_ejemplo(self, nombre):
        contenido = self._obtener_codigo_del_ejemplo(nombre)
        self.ui.codigo.document().setPlainText(contenido)

    def _mostrar_codigo_presentacion_inicial(self):
        self.ui.codigo.document().setPlainText(MENSAJE_PRESENTACION)

    def _obtener_codigo_del_ejemplo(self, nombre):
        archivo = open(nombre)
        contenido = archivo.read()
        archivo.close()
        return contenido

    def _obtener_item_actual(self):
        return self.ui.arbol.currentItem().text(1)

    def _ejecutar_ejemplo(self, ruta):
        self.process = QtCore.QProcess(self)
        self.process.setProcessChannelMode(QtCore.QProcess.MergedChannels)
        self.process.finished.connect(
            self._cuando_termina_la_ejecucion_del_ejemplo)
        self.process.start(sys.executable, [ruta])

        # Deshabilita todos los controles para que se pueda
        # ejecutar un ejemplo a la vez.
        self._definir_estado_habilitado(False)

    def _cuando_termina_la_ejecucion_del_ejemplo(self, estado, process):
        "Vuelve a permitir que se usen todos los botone de la interfaz."
        print self.process.readAll()
        self._definir_estado_habilitado(True)
        self.ui.arbol.setFocus()


def main():
    app = QtGui.QApplication(sys.argv)
    window = VentanaPrincipal()
    window.show()
    sys.exit(app.exec_())


if __name__ == "__main__":
    main()



import pilas
pilas.iniciar(gravedad=(0,0))
mapa = pilas.actores.Mapa('parque.tmx')

# Genera un personaje en movimiento.
aceituna = pilas.actores.Aceituna()
circulo = pilas.fisica.Circulo(0, 0, 14, restitucion=1, friccion=0)
aceituna.imitar(circulo)
circulo.impulsar(20000, 20000)

pilas.avisar("Cargando el mapa desde un archivo (F11 muestra colisiones)")
pilas.ejecutar()



import pilas

def ejecutar():
    globo = pilas.actores.Globo("Hola mundo")
    pilas.avisar("Mostrando un mensaje sencillo")


if __name__ == '__main__':
    pilas.iniciar()
    ejecutar()
    pilas.ejecutar()



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")

pilas.iniciar()
boton = pilas.interfaz.Boton("Hola mundo")

def cuando_hacen_click():
    boton.decir("Has pulsado el boton!")

boton.conectar(cuando_hacen_click)
pilas.avisar("Creando un boton con texto.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# For pilas engine - a video game framework.
#
# copyright 2011 - Pablo Garrido
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pilas
from random import random

class puntuacion:
    p = 0

class Juego(pilas.escenas.Escena):
    "Escena que Controla el juego"

    def __init__(self, nivel):
        puntuacion.p = 0 
        self.iniciar_juego(nivel)
        
        
    
    def iniciar_juego(self, nivel):

        pilas.escenas.Escena.__init__(self) 
        pilas.actores.utils.eliminar_a_todos()
        pilas.fondos.Fondo('data/fondo.png')

        if nivel == 1:
            self.pos_iniciar = [-35, -35]
            self.filas = 2
            self.columnas = 2
            self.espaciado = 30
            self.pares = 2
            self.agregar_puntaje = 5
            self.restar_puntaje = 1

        elif nivel == 2:
            self.pos_iniciar = [-50, -35]
            self.filas = 3
            self.columnas = 2
            self.espaciado = 15
            self.pares = 3
            self.agregar_puntaje = 10
            self.restar_puntaje = 2

        elif nivel == 3:
            self.pos_iniciar = [-78, -65]
            self.filas = 4
            self.columnas = 3
            self.espaciado = 15
            self.pares = 6
            self.agregar_puntaje = 15
            self.restar_puntaje = 3

        elif nivel == 4:
            self.pos_iniciar = [-70, -85]
            self.filas = 4
            self.columnas = 4
            self.espaciado = 10
            self.pares = 8
            self.agregar_puntaje = 20
            self.restar_puntaje = 4

        elif nivel == 5:
            self.pos_iniciar = [-93, -85]
            self.filas = 5
            self.columnas = 4
            self.espaciado = 10
            self.pares = 10
            self.agregar_puntaje = 25
            self.restar_puntaje = 5

        elif nivel == 6:
            self.pos_iniciar = [-118, -85]
            self.filas = 6
            self.columnas = 4
            self.espaciado = 10
            self.pares = 12
            self.agregar_puntaje = 30
            self.restar_puntaje = 5
            
        elif nivel == 7:
            self.pos_iniciar = [-105, -155]
            self.filas = 6
            self.columnas = 6
            self.espaciado = 5
            self.pares = 18
            self.agregar_puntaje = 35
            self.restar_puntaje = 6
  
        elif nivel == 8:
            self.pos_iniciar = [-130, -150]
            self.filas = 7
            self.columnas = 6
            self.espaciado = 5
            self.pares = 21
            self.agregar_puntaje = 100
            self.restar_puntaje = 7
         
        pilas.eventos.pulsa_tecla_escape.conectar(self.cuando_se_presione_escape)
        
        
        self.crear_puntaje()
        self.crear_nivel(nivel)
        self.crear_parejas()

        self.nivel = nivel
        self.estado = True

        self.tiempo = 0.5
        
        self.area = self.columnas * self.filas

        self.intentos = []
        self.cantidad_de_pares = 0

        self.array_caracteres = self.generar_array_caracteres()
        
        self.fabricar_botones_personalizados()
        self.configurar_botones()


    def cambiar_nivel(self, nivel):
        self.nivel = nivel
        self.iniciar_juego(self.nivel)
            
    def fabricar_botones_personalizados(self):
        self.botones = pilas.atajos.fabricar(boton_personalizado, self.area, False)

    def cuando_se_presione_escape(self, *k, **kv):
        "Regresa al menu principal"
        import escena_menu
        escena_menu.EscenaMenu()


    def crear_puntaje(self):
        self.puntaje = pilas.actores.Puntaje()
        self.puntaje.definir(puntuacion.p)
        self.puntaje.x = -200
        self.puntaje.y = 55
        self.puntaje.magnitud = 18
        
    def crear_parejas(self):
        self.parejas = pilas.actores.Puntaje()
        self.parejas.definir(0)
        self.parejas.x = -200
        self.parejas.y = 17
        self.parejas.magnitud = 18
       
    def crear_nivel(self, nivel):
        self.nivel_texto = pilas.actores.Puntaje()
        self.nivel_texto.x = -200
        self.nivel_texto.y = 93
        self.nivel_texto.magnitud = 18
        self.nivel_texto.definir(nivel)

        


    def generar_array_caracteres(self):
        # lista con caracteres

        # lista con caracteres aleatorios
        caracteres_aleatorios = []
        for c_a in range(self.area):
            caracteres_aleatorios.append("null")

        # lista con letras 
        l = []
        for c in "abcdefghijklmnopqrstu":
            l.append(c)
        
        

        # creamos una lista 'caracteres_aleatorios' con letras
        # desordenadas
    
        caracteres = []
        limite = 0
        limite = self.pares


        for i in range(limite):
            caracteres.append(l[i])
            caracteres.append(l[i] + '2')


        for i in range(self.area):
            ocupado = True
            while ocupado:
                rand_i = int(random() * self.area)
                if caracteres_aleatorios[rand_i] == "null":
                    caracteres_aleatorios[rand_i] = caracteres[i]
                    ocupado = False

        return caracteres_aleatorios

        
        

    def pintar_boton_x(self, n):
        if self.estado == True:
            if n.primera_vez_presionado == False:
            
                for c in 'abcdefghijklmnopqrstu':
                    if n.apariencia == c:
                        self.intentos.append([c, 1])                    
                        n.boton.pintar_presionado(c + '.png')  

                    if n.apariencia == c + '2':
                        self.intentos.append([c, 2])
                        n.boton.pintar_presionado(c + '.png')               
                
        
            n.primera_vez_presionado = True

    def termino_juego(self):
        self.termino = pilas.actores.Actor("termino.png")
        self.termino.y = 300
        self.termino.y = pilas.interpolar(0, 2)

        self.p = pilas.actores.Puntaje()
        self.p.definir(puntuacion.p)
        
        self.p.y = 280
        self.p.x = -10
        self.p.y = pilas.interpolar(-20, 2)
        
        self.estado = False

    
    def analizar_pares(self):

        def aumentar_pares():
            self.cantidad_de_pares += 1
            self.parejas.definir(self.cantidad_de_pares)
            puntuacion.p += self.agregar_puntaje
            self.puntaje.definir(puntuacion.p)

        def borrar_ficha_x(ficha_x, ficha_x2):
            ficha_x.boton.eliminar()
            ficha_x.boton.desactivar()

            ficha_x2.boton.eliminar()
            ficha_x2.boton.desactivar()
            aumentar_pares()

            if self.cantidad_de_pares == self.pares:

                self.nivel += 1
                if self.nivel > 8:
                    self.termino_juego()
                else:
                    self.cambiar_nivel(self.nivel)                
            self.estado = True

        def encontrar_boton_x(c):
            x = 0
            for i in self.array_caracteres:
                if self.botones[x].apariencia == c:
                   return self.botones[x]
                x += 1
            
        
        if (len(self.intentos))  == 2:
            # si son pares
            if self.intentos[0][0] == self.intentos[1][0]:
                if self.estado == True:
                    for c in 'abcdefghijklmnopqrstu': 
                        if self.intentos[0][0] == c: 
                            boton_x = encontrar_boton_x(c)
                            boton_x2 = encontrar_boton_x(c + '2')
                            pilas.mundo.agregar_tarea_una_vez(self.tiempo, borrar_ficha_x, boton_x, boton_x2)

                self.estado = False
            else:
            # si no son pares

                puntuacion.p -= self.restar_puntaje
                self.puntaje.definir(puntuacion.p)

                def pintar_normal_ficha_x(n):
                    n.boton.pintar_normal()
                    self.estado = True

                for i in range(self.area):
                    self.botones[i].primera_vez_presionado = False
                    self.estado = False
                    pilas.mundo.agregar_tarea_una_vez(self.tiempo, pintar_normal_ficha_x, self.botones[i])
                
            self.intentos.pop()
            self.intentos.pop()

    def configurar_botones(self):
        x = 0

        for i in range(self.columnas):
            for j in range(self.filas):        
                
                self.botones[x].boton.x = self.pos_iniciar[0] 
                self.botones[x].boton.y = self.pos_iniciar[1]

                # posicionamos botones en filas y columnas
                self.botones[x].boton.x += (self.botones[x].boton.obtener_ancho() + self.espaciado) * j     
                self.botones[x].boton.y += (self.botones[x].boton.obtener_alto() + self.espaciado) * i 

                # agregamos identidad al boton
                self.botones[x].apariencia = self.array_caracteres[x] 
       
                # conectamos funcion
                self.botones[x].boton.conectar_presionado(self.pintar_boton_x, self.botones[x])
                self.botones[x].boton.conectar_presionado(self.analizar_pares)

                x += 1    


class boton_personalizado:
    def __init__(self, x = 0, y = 0):

        ruta_normal = "casilla.png"        

        self.boton = pilas.actores.Boton(x, y, ruta_normal)
        self.apariencia = 'null'
        self.primera_vez_presionado = False



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Actor
from pilas import colores
            
class Pizarra(Actor):
    """Representa una superficie de dibujo inicialmente transparente.

    Puedes pintar sobre esta pizarra usando métodos que simulan
    un lapiz, que se puede mover sobre una superficie.
    """

    def __init__(self, x=0, y=0, ancho=None, alto=None):
        # Si no define area de la pizarra toma el tamano de la ventana.
        if not ancho or not alto:
            ancho, alto = pilas.mundo.motor.obtener_area()

        Actor.__init__(self, x=x, y=y)
        self.imagen = pilas.imagenes.cargar_superficie(ancho, alto)

    def dibujar_punto(self, x, y, color=colores.negro):
        x, y = self.obtener_coordenada_fisica(x, y)
        self.imagen.dibujar_punto(x, y, color=color)

    def obtener_coordenada_fisica(self, x, y):
        x = (self.imagen.ancho()/2) + x
        y = (self.imagen.alto()/2) - y
        return x, y

    def pintar_imagen(self, imagen, x, y):
        self.pintar_parte_de_imagen(imagen, 0, 0, imagen.ancho(), imagen.alto(), x, y)

    def pintar_parte_de_imagen(self, imagen, origen_x, origen_y, ancho, alto, x, y):
        x, y = self.obtener_coordenada_fisica(x, y)
        self.imagen.pintar_parte_de_imagen(imagen, origen_x, origen_y, ancho, alto, x, y)

    def pintar_grilla(self, grilla, x, y):
        grilla.dibujarse_sobre_una_pizarra(self, x, y)

    def pintar(self, color):
        self.imagen.pintar(color)

    def linea(self, x, y, x2, y2, color=colores.negro, grosor=1):
        x, y = self.obtener_coordenada_fisica(x, y)
        x2, y2 = self.obtener_coordenada_fisica(x2, y2)
        self.imagen.linea(x, y, x2, y2, color, grosor)

    def rectangulo(self, x, y, ancho, alto, color=colores.negro, relleno=False, grosor=1):
        x, y = self.obtener_coordenada_fisica(x, y)
        self.imagen.rectangulo(x, y, ancho, alto, color, relleno, grosor)

    def texto(self, cadena, x=0, y=0, magnitud=10, fuente=None, color=colores.negro):
        x, y = self.obtener_coordenada_fisica(x, y)
        self.imagen.texto(cadena, x, y, magnitud, fuente, color)

    def poligono(self, puntos, color=pilas.colores.negro, grosor=1):
        puntos = [self.obtener_coordenada_fisica(*p) for p in puntos]
        self.imagen.poligono(puntos, color, grosor)



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")

pilas.iniciar()
pizarra = pilas.actores.Pizarra()

grilla = pilas.imagenes.cargar_grilla("pingu.png", 10)
pizarra.pintar_grilla(grilla, 0, 0)

grilla.definir_cuadro(2)
pizarra.pintar_grilla(grilla, 100, 100)

grilla.definir_cuadro(3)
pizarra.pintar_grilla(grilla, 200, 200)

pilas.avisar("Muestra algunos cuadros de animacion sobre una pizarra")
pilas.ejecutar()



import pilas
pilas.iniciar()


t = pilas.actores.Temporizador()

def funcion_callback():
    pilas.avisar("Temporizador: el tiempo se acabo!")

# ajustamos que despues de 3 segundos llame a funcion_callback
t.ajustar(10, funcion_callback)

# iniciamos temporizador
t.iniciar()


pilas.ejecutar()



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")

pilas.iniciar()
t = pilas.actores.Pingu()

pilas.avisar("Usa el teclado para mover al personaje (para arriba 'salta').")
pilas.ejecutar()



# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")
import pilas

pilas.iniciar()
actor = pilas.actores.Actor()


pilas.avisar("Este es un ejemplo de actor creado pero sin imagen.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar


import math

import pilas
import utils
from actor import Actor

todos = []

__doc__ = """
Módulo pilas.actores
====================

El módulo actores contiene una serie de clases
para representar personajes de videojuegos.

Para crear actores en una escena del juego simplemente
se tiene que crear un nuevo objeto a partir de una
clase.

Por ejemplo, para crear un pongüino podríamos
escribir la siguiente sentencia:

    >>> p = pilas.actores.Pingu()

"""



from mono import Mono
from ejes import Ejes
from animado import Animado
from animacion import Animacion
from explosion import Explosion
from bomba import Bomba
from pingu import Pingu
from banana import Banana
from texto import Texto
from temporizador import Temporizador
from moneda import Moneda
from pizarra import Pizarra
from pelota import Pelota
from puntaje import Puntaje
from estrella import Estrella
from caja import Caja
from nave import Nave
from disparo import Disparo
from cursordisparo import CursorDisparo
from piedra import Piedra
from menu import Menu
from opcion import Opcion
from tortuga import Tortuga
from mapa import Mapa
from martian import Martian
from boton import Boton
from entradadetexto import EntradaDeTexto
from aceituna import Aceituna
from globo import Globo
from dialogo import Dialogo
from globoelegir import GloboElegir
from pausa import Pausa
from mano import CursorMano



import pilas
pilas.iniciar()

selector = pilas.interfaz.Selector("Usar fondo", x=0, y=0)
fondo = None

def cuando_el_selector_cambia(estado):
    global fondo

    if estado:
        fondo = pilas.fondos.Pasto()
    else:
        fondo.eliminar()


selector.definir_accion(cuando_el_selector_cambia) 

pilas.ejecutar()



import weakref

from pilas.dispatch import saferef

WEAKREF_TYPES = (weakref.ReferenceType, saferef.BoundMethodWeakref)


class DictObj(object):
    def __init__(self, d):
        self.d = d

    def __getattr__(self, m):
        return self.d.get(m, None)
    
    def __str__(self):
        return "<Evento %s>" %(str(self.d))

def _make_id(target):
    if hasattr(target, 'im_func'):
        return (id(target.im_self), id(target.im_func))
    return id(target)

class Signal(object):
    """
    Base class for all signals
    
    Internal attributes:
    
        receivers
            { receriverkey (id) : weakref(receiver) }
    """
    
    def __init__(self, providing_args=None):
        """
        Create a new signal.
        
        providing_args
            A list of the arguments this signal can pass along in a send() call.
        """
        self.receivers = []
        if providing_args is None:
            providing_args = []
        self.providing_args = set(providing_args)

    def conectar(self, receptor, emisor=None, weak=True, uid=None):
        return self.connect(receptor, emisor, weak, uid)

    def connect(self, receiver, sender=None, weak=True, uid=None):
        """
        Connect receiver to sender for signal.
    
        Arguments:
        
            receiver
                A function or an instance method which is to receive signals.
                Receivers must be hashable objects.

                if weak is True, then receiver must be weak-referencable (more
                precisely saferef.safeRef() must be able to create a reference
                to the receiver).
        
                Receivers must be able to accept keyword arguments.

                If receivers have a uid attribute, the receiver will
                not be added if another receiver already exists with that
                uid.

            sender
                The sender to which the receiver should respond Must either be
                of type Signal, or None to receive events from any sender.

            weak
                Whether to use weak references to the receiver By default, the
                module will attempt to use weak references to the receiver
                objects. If this parameter is false, then strong references will
                be used.
        
            uid
                An identifier used to uniquely identify a particular instance of
                a receiver. This will usually be a string, though it may be
                anything hashable.
        """
        
        if uid:
            lookup_key = (uid, _make_id(sender))
        else:
            lookup_key = (_make_id(receiver), _make_id(sender))

        if weak:
            receiver = saferef.safeRef(receiver, onDelete=self._remove_receiver)

        for r_key, _ in self.receivers:
            if r_key == lookup_key:
                break
        else:
            self.receivers.append((lookup_key, receiver))

    def desconectar(self, receptor=None, emisor=None, weak=True, uid=None):
        self.disconnect(receptor, emisor, weak, uid)

    def disconnect(self, receiver=None, sender=None, weak=True, uid=None):
        """
        Disconnect receiver from sender for signal.

        If weak references are used, disconnect need not be called. The receiver
        will be remove from dispatch automatically.
    
        Arguments:
        
            receiver
                The registered receiver to disconnect. May be none if
                uid is specified.
            
            sender
                The registered sender to disconnect
            
            weak
                The weakref state to disconnect
            
            uid
                the unique identifier of the receiver to disconnect
        """
        if uid:
            lookup_key = (uid, _make_id(sender))
        else:
            lookup_key = (_make_id(receiver), _make_id(sender))
        
        for index in xrange(len(self.receivers)):
            (r_key, _) = self.receivers[index]
            if r_key == lookup_key:
                del self.receivers[index]
                break

    def send(self, sender, **named):
        """
        Send signal from sender to all connected receivers.

        If any receiver raises an error, the error propagates back through send,
        terminating the dispatch loop, so it is quite possible to not have all
        receivers called if a raises an error.

        Arguments:
        
            sender
                The sender of the signal Either a specific object or None.
    
            named
                Named arguments which will be passed to receivers.

        Returns a list of tuple pairs [(receiver, response), ... ].
        """
        responses = []
        if not self.receivers:
            return responses

        for receiver in self._live_receivers(_make_id(sender)):
            response = receiver(DictObj(named))
            responses.append((receiver, response))
        return responses

    def send_robust(self, sender, **named):
        """
        Send signal from sender to all connected receivers catching errors.

        Arguments:
        
            sender
                The sender of the signal Can be any python object (normally one
                registered with a connect if you actually want something to
                occur).

            named
                Named arguments which will be passed to receivers. These
                arguments must be a subset of the argument names defined in
                providing_args.

        Return a list of tuple pairs [(receiver, response), ... ]. May raise
        DispatcherKeyError.

        if any receiver raises an error (specifically any subclass of
        Exception), the error instance is returned as the result for that
        receiver.
        """
        responses = []
        if not self.receivers:
            return responses

        # Call each receiver with whatever arguments it can accept.
        # Return a list of tuple pairs [(receiver, response), ... ].
        for receiver in self._live_receivers(_make_id(sender)):
            try:
                response = receiver(signal=self, sender=sender, **named)
            except Exception, err:
                responses.append((receiver, err))
            else:
                responses.append((receiver, response))
        return responses

    def _live_receivers(self, senderkey):
        """
        Filter sequence of receivers to get resolved, live receivers.

        This checks for weak references and resolves them, then returning only
        live receivers.
        """
        none_senderkey = _make_id(None)
        receivers = []

        for (receiverkey, r_senderkey), receiver in self.receivers:
            if r_senderkey == none_senderkey or r_senderkey == senderkey:
                if isinstance(receiver, WEAKREF_TYPES):
                    # Dereference the weak reference.
                    receiver = receiver()
                    if receiver is not None:
                        receivers.append(receiver)
                else:
                    receivers.append(receiver)
        return receivers

    def _remove_receiver(self, receiver):
        """
        Remove dead receivers from connections.
        """

        to_remove = []
        for key, connected_receiver in self.receivers:
            if connected_receiver == receiver:
                to_remove.append(key)
        for key in to_remove:
            for idx, (r_key, _) in enumerate(self.receivers):
                if r_key == key:
                    del self.receivers[idx]

    def esta_conectado(self):
        "Indica si tiene alguna funcion conectada."
        return self.receivers
    
    def imprimir_funciones_conectadas(self):
        "Imprime todas las funciones que tiene conectado el evento."
        for clave, referencia in self.receivers:
            nombre = referencia.__str__()
            nombre = nombre[nombre.index('(')+1:nombre.index(")")]
            print "\t", nombre


import pilas

class GirarPorSiempre(pilas.habilidades.Habilidad):
    
    def __init__(self, receptor, velocidad=1):
        self.receptor = receptor
        self.velocidad = velocidad
    
    def actualizar(self):
        self.receptor.rotacion += self.velocidad

pilas.iniciar()

a = pilas.actores.Mono()
a.aprender(GirarPorSiempre, 20)
pilas.avisar("Girando por siempre...")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
import pilas

pilas.iniciar()                                                                                         

# La secuencia es sencilla, el mono dice 'hola', 
mono = pilas.actores.Mono(x=-100)
mono_chiquito = pilas.actores.Mono(x=200)
mono_chiquito.escala = 0.75
d = pilas.actores.Dialogo()
d.decir(mono, "¿Cual es tu color favorito?")

def cuando_responde_color_favorito(respuesta):
    colores = {
               'rojo': pilas.colores.rojo,
               'verde': pilas.colores.verde,
               'azul': pilas.colores.azul,
              }

    pilas.fondos.Color(colores[respuesta])
    mono.sonreir()
    d.decir(mono, "¡mira!")
    d.decir(mono_chiquito, "fua...")

d.elegir(mono_chiquito, "Mi color favorito es el...", ["rojo", "verde", "azul"], cuando_responde_color_favorito)


d.iniciar()
pilas.avisar("Tienes que hacer click para que la animacion avance.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

from pilas.actores import Actor
import pilas

class CursorMano(Actor):
    "Representa un bloque que tiene fisica como una caja."

    def __init__(self, x=0, y=0):
        self._cargar_imagenes()
        Actor.__init__(self, self.imagen_normal)
        self.x = x
        self.y = y

        self.aprender(pilas.habilidades.SeguirAlMouse)
        pilas.mundo.motor.ocultar_puntero_del_mouse()
        self.z = -200
        self.pulsado = False
        
        self.centro = ("izquierda", "arriba")

        pilas.eventos.mueve_mouse.conectar(self.cuando_mueve_el_mouse)
        pilas.eventos.click_de_mouse.conectar(self.cuando_pulsa_el_mouse)
        pilas.eventos.termina_click.conectar(self.cuando_suelta_el_mouse)

    def _cargar_imagenes(self):
        self.imagen_normal = pilas.imagenes.cargar("cursores/normal.png")
        self.imagen_arrastrando = pilas.imagenes.cargar("cursores/arrastrando.png")

    def cuando_pulsa_el_mouse(self, evento):
        self.pulsado = True

    def cuando_mueve_el_mouse(self, evento):
        if self.pulsado:
            self.imagen = self.imagen_arrastrando

    def cuando_suelta_el_mouse(self, evento):
        if self.pulsado:
            self.imagen = self.imagen_normal
            self.pulsado = False



# -*- encoding: utf-8 -*-
import pilas
from pilas.actores.globo import Globo

class GloboElegir(Globo):
    
    def __init__(self, texto, opciones, funcion_a_invocar, x=0, y=0, dialogo=None):
        self.dialogo = dialogo
        self.opciones = opciones
        self.funcion_a_invocar = funcion_a_invocar
        espacio = "\n" * (len(opciones) +1)   # representa un espacio en blanco para poner la seleccion
        Globo.__init__(self, texto + espacio, x, y, dialogo=dialogo)
    
        self.lista_seleccion = pilas.interfaz.ListaSeleccion(opciones, self._cuando_selecciona_opcion, x, y)
        self.lista_seleccion.escala = 0.1
        self.lista_seleccion.escala = [1], 0.2
        
    def colocar_origen_del_globo(self, x, y):
        self.lista_seleccion.centro = ("derecha", "abajo")
        self.lista_seleccion.x = x - 10
        self.lista_seleccion.y = y - 10

    def _obtener_area_para_el_texto(self, texto):
        ancho, alto = self.lienzo.obtener_area_de_texto(texto, tamano=14)
        opciones_ancho, opciones_alto = self.lienzo.obtener_area_para_lista_de_texto(self.opciones, tamano=14)     
        return ancho + opciones_ancho, alto + opciones_alto 
        
    def _escribir_texto(self, texto):
        self.lienzo.escribir(texto, 12, 25, tamano=14)

    def cuando_quieren_avanzar(self, *k):
        pass

    def _cuando_selecciona_opcion(self, opcion):
        self.funcion_a_invocar(opcion)
        Globo.cuando_quieren_avanzar(self)
        self.lista_seleccion.eliminar()



import pilas

class ContadorDeVidas:

    def __init__(self, cantidad_de_vidas):
        self.crear_texto()
        self.cantidad_de_vidas = cantidad_de_vidas
        self.vidas = [pilas.actores.Actor("data/vida.png") for x in range(cantidad_de_vidas)]

        for indice, vida in enumerate(self.vidas):
            vida.x = -230 + indice * 30
            vida.arriba = 230

    def crear_texto(self):
        "Genera el texto que dice 'vidas'"
        self.texto = pilas.actores.Texto("Vidas:")
        self.texto.color = pilas.colores.blanco
        self.texto.magnitud = 20
        self.texto.izquierda = -310
        self.texto.arriba = 220

    def le_quedan_vidas(self):
        return self.cantidad_de_vidas > 0

    def quitar_una_vida(self):
        self.cantidad_de_vidas -= 1
        vida = self.vidas.pop()
        vida.eliminar()




# -*- encoding: utf-8 -*-
# For Pilas engine - A video game framework.
#
# Copyright 2010 - Pablo Garrido
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar
#

import pilas
from pilas.actores import Texto
from pilas import colores

class Temporizador(Texto):
    def __init__(self, x=0, y=0, color=colores.negro):
        Texto.__init__(self, '0', x=x, y=y)
        self.ajustar(1, self.funcion_vacia)
        self.color = color


    # funcion cuando no se ajusta temporizador   
    def funcion_vacia(self):
        pass
    

    def definir_tiempo_texto(self, variable):
        self.texto = str(variable)


    # con la funcion ajustar manipulamos el tiempo y la
    # funcion queremos ejecutar
    def ajustar(self, tiempo = 1, funcion = None):

        self.tiempo = tiempo 
        self.definir_tiempo_texto(self.tiempo)

        if funcion == None:
            self.funcion = self.funcion_vacia()
        else:
            self.funcion = funcion



    # resta -1 hasta llegar a 0 el contador (numero en patalla)
    def restar_a_contador(self):
        if self.tiempo != 0:
            self.tiempo -= 1
            self.definir_tiempo_texto(self.tiempo)
            return True
   
    

    # iniciamos temporizador con respectivas funciones y tiempo ajustado
    def iniciar(self):
        pilas.mundo.agregar_tarea_una_vez(self.tiempo, self.funcion)
        pilas.mundo.agregar_tarea_siempre(1, self.restar_a_contador)
        
        
        
    
    





# -*- encoding: utf-8 -*-
import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")


pilas.iniciar()
pilas.fondos.Pasto()

# Genera un texto que dice "bienvenido a pilas"
saludo = pilas.actores.Texto(u"¡Bienvenido a pilas!")

# Realiza una animacion
saludo.escala = 0.1
saludo.escala = [1]
saludo.rotacion = [360]

saludo.aprender(pilas.habilidades.MoverseConElTeclado)

pilas.avisar("Ejemplo de texto, pulse las teclas para desplazar el texto.")
pilas.ejecutar()



import pilas

class Color(object):
    "Representa un color en base a 4 componentes."

    def __init__(self, r, g, b, a=255):
        self.r = r
        self.g = g
        self.b = b
        self.a = a

    def obtener(self):
        return pilas.motor.Color(self.r, self.g, self.b, self.a)

    def __str__(self):
        return "<Color (%d, %d, %d, %d)>" %(self.r, self.g, self.b, self.a)

    def obtener_componentes(self):
        return (self.r, self.g, self.b, self.a)

# Colores principales.
negro = Color(0, 0, 0)
blanco = Color(255, 255, 255)
rojo = Color(255, 0, 0)
verde = Color(0, 255, 0)
azul = Color(0, 0, 255)
gris = Color(128, 128, 128)

# Colores secundarios
amarillo = Color(255, 255, 0)
magenta = Color(255, 0, 255)
cyan = Color(0, 255, 255)
grisclaro = Color(192, 192, 192)
grisoscuro = Color(100, 100, 100)
verdeoscuro = Color(0, 128, 0)
azuloscuro = Color(0, 0, 128)
naranja = Color(255, 200, 0)
rosa = Color(255, 175, 175)
violeta = Color(128, 0, 255)
marron = Color(153, 102, 0)

# Colores transparentes
negro = Color(0, 0, 0, 160)
blanco = Color(255, 255, 255, 160)
rojo_transparente = Color(255, 0, 0, 160)
verde_transparente = Color(0, 255, 0, 160)
azul_transparente = Color(0, 0, 255, 160)
gris_transparente = Color(128, 128, 128, 160)



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
import re

class IngresoDeTexto(pilas.actores.Actor):
    
    def __init__(self, texto_inicial="", x=0, y=0, ancho=300, limite_de_caracteres=20, icono=None):
        pilas.actores.Actor.__init__(self, x=x, y=y)
        self.texto = texto_inicial
        self.cursor = ""
        self._cargar_lienzo(ancho)

        if icono:
            self.icono = pilas.imagenes.cargar(icono)
        else:
            self.icono = None

        self.imagen_caja = pilas.imagenes.cargar("interfaz/caja.png")
        self.centro = ("centro", "centro")
        self._actualizar_imagen()
        self.limite_de_caracteres = limite_de_caracteres
        self.cualquier_caracter()

        pilas.eventos.pulsa_tecla.conectar(self.cuando_pulsa_una_tecla)
        pilas.mundo.agregar_tarea_siempre(0.40, self._actualizar_cursor)
        
    def _actualizar_cursor(self):
        if self.cursor == "":
            self.cursor = "_"
        else:
            self.cursor = ""
            
        self._actualizar_imagen()
        return True

    def cualquier_caracter(self):
        self.caracteres_permitidos = re.compile(".*")

    def solo_numeros(self):
        self.caracteres_permitidos = re.compile("\d+")

    def solo_letras(self):
        self.caracteres_permitidos = re.compile("[a-z]+")
        
    def cuando_pulsa_una_tecla(self, evento):
        if evento.codigo == '\x08' or evento.texto == '\x08':
            # Indica que se quiere borrar un caracter
            self.texto = self.texto[:-1]
        else:
            if len(self.texto) < self.limite_de_caracteres:
                nuevo_texto = self.texto + evento.texto

                if (self.caracteres_permitidos.match(evento.texto)):
                    self.texto = self.texto + evento.texto
                else:
                    print "Rechazando el ingreso del caracter:", evento.texto
            else:
                print "Rechazando caracter por llegar al limite."
        
        self._actualizar_imagen()
        
    def _cargar_lienzo(self, ancho):
        self.imagen = pilas.imagenes.cargar_superficie(ancho, 30)
        
    def _actualizar_imagen(self):
        ancho = self.imagen_caja.ancho()
        alto = self.imagen_caja.alto()
        self.imagen.pintar_parte_de_imagen(self.imagen_caja, 0, 0, 40, ancho, 0, 0)

        if self.icono:
            dx = 20
            self.imagen.pintar_parte_de_imagen(self.icono, 0, 0, 40, ancho, 7, 7)
        else:
            dx = 0

        for x in range(40, self.imagen.ancho() - 40):
            self.imagen.pintar_parte_de_imagen(self.imagen_caja, ancho - 40, 0, 40, alto, x, 0)

        self.imagen.texto(self.texto + self.cursor, 15 + dx, 20)



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")


pilas.iniciar()
pizarra = pilas.actores.Pizarra()

pizarra.linea(0, 0, 100, 0, grosor=2)
pizarra.linea(100, 0, 100, 100, grosor=2)
pizarra.linea(100, 100, 0, 0, grosor=2)


pilas.avisar("Una pizarra con algunas lineas.")
pilas.ejecutar()



import pilas

# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")

pilas.iniciar()

# Dos pinguinos:
pingu1 = pilas.actores.Pingu(x=-100)
pingu2 = pilas.actores.Pingu(x=+100)

# El pinguino de la derecha tendra el punto de control en los pies.
pingu2.centro = ('centro', 'arriba')

pilas.avisar("Dos actores con distintos centros (puntos de control), pulsa F12.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from actor import Actor

class Texto(Actor):
    """Representa un texto en pantalla.

    El texto tiene atributos como ``texto``, ``magnitud`` y ``color``.
    """
    
    def __init__(self, texto="None", x=0, y=0, magnitud=20):
        imagen = pilas.mundo.motor.obtener_texto(texto, magnitud)
        self._definir_area_de_texto(texto, magnitud)
        Actor.__init__(self, imagen, x=x, y=y)
        self.magnitud = magnitud
        self.texto = texto
        self.color = pilas.colores.blanco
        self.centro = ("centro", "centro")
        self.fijo = True

    def obtener_texto(self):
        return self.imagen.texto

    def definir_texto(self, texto):
        self.imagen.texto = texto
        self._definir_area_de_texto(texto, self.magnitud)

    texto = property(obtener_texto, definir_texto, doc="El texto que se tiene que mostrar.")
    
    def obtener_magnitud(self):
        return self.imagen.magnitud
    
    def definir_magnitud(self, magnitud):
        self._magnitud = magnitud
        self.imagen.magnitud = magnitud
    
    magnitud = property(obtener_magnitud, definir_magnitud, doc="El tamaño del texto.")
    
    def obtener_color(self):
        return self.imagen.color
    
    def definir_color(self, color):
        self.imagen.color = color

    color = property(obtener_color, definir_color, doc="Color del texto.")

    def _definir_area_de_texto(self, texto, magnitud):
        self._ancho, self._alto = pilas.mundo.motor.obtener_area_de_texto(texto, magnitud)



# -*- encoding: utf-8 -*-
# For Pilas engine - A video game framework.
#
# Copyright 2010 - Pablo Garrido
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar
#


import pilas
from pilas.actores import Texto

class Puntaje(Texto):
    """Representa un contador de Puntaje"""
	
    def __init__(self, texto='0', x=0, y=0, color=pilas.colores.negro):
        Texto.__init__(self, texto, x=x, y=y)
        self.color = color

    def definir(self, puntaje_variable = '0'):
        self.puntaje_texto = str(puntaje_variable)
        self.texto = self.puntaje_texto

    def aumentar(self, cantidad=1):
        self.definir(int(self.texto) + int(cantidad))
        
    def obtener(self):
        return int(self.texto)




import pilas
import random

class PiedraEspacial(pilas.actores.Piedra):
    "Representa una piedra espacial que se puede romper con un disparo."

    def __init__(self, piedras, x=0, y=0, tamano="grande"):
        # Obtiene una velocidad de movimiento aleatoria.
        posibles_velocidades = range(-10, -2) + range(2, 10)

        dx = random.choice(posibles_velocidades) / 10.0
        dy = random.choice(posibles_velocidades) / 10.0

        pilas.actores.Piedra.__init__(self, x=x, y=y, dx=dx, dy=dy, tamano=tamano)
        self.tamano = tamano
        self.piedras = piedras

    def eliminar(self):
        "Este metodo se invoca cuando el disparo colisiona con la nave."
        explosion = pilas.actores.Explosion(self.x, self.y)
        pilas.actores.Piedra.eliminar(self)

        if self.tamano == "grande":
            self.crear_dos_piedras_mas_pequenas(self.x, self.y, "media")
        elif self.tamano == "media":
            self.crear_dos_piedras_mas_pequenas(self.x, self.y, "chica")

    def crear_dos_piedras_mas_pequenas(self, x, y, tamano):
        "Genera dos piedras mas chicas, enemigas de la nave."
        piedra_1 = PiedraEspacial(self.piedras, x=x, y=y, tamano=tamano)
        piedra_2 = PiedraEspacial(self.piedras, x=x, y=y, tamano=tamano)
        self.piedras.append(piedra_1)
        self.piedras.append(piedra_2)



# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")
import pilas

pilas.iniciar()

grilla = pilas.imagenes.cargar_grilla("grillas/plataformas_10_10.png", 10, 10)
mapa = pilas.actores.Mapa(grilla)

mapa.pintar_bloque(10, 10, 0, True)
mapa.pintar_bloque(10, 11, 1, True)
mapa.pintar_bloque(10, 12, 1, True)
mapa.pintar_bloque(10, 13, 1, True)
mapa.pintar_bloque(10, 14, 2, True)


mapa.pintar_bloque(10, 0, 0, True)
mapa.pintar_bloque(10, 1, 1, True)
mapa.pintar_bloque(10, 2, 1, True)
mapa.pintar_bloque(10, 3, 1, True)
mapa.pintar_bloque(10, 4, 2, True)

pelotas = pilas.atajos.fabricar(pilas.actores.Pelota, 10)

pilas.avisar("Creando dos plataformas solidas...")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.simbolos import *
import eventos

__doc__ = """
Módulo pilas.control
====================

"""

class Control(object):
    """Representa un control de teclado sencillo.

    Este objeto permite acceder al estado del teclado usando
    atributos.

    Por ejemplo, con este objeto, para saber si el usuario
    está pulsando el direccional hacia la izquierda de
    puedes ejecutar::

        if pilas.mundo.control.izquierda:
            print 'Ha pulsado hacia la izquierda'

    Es decir, si bien Control es una clase, no hace falta
    instanciarla. Ya existe un objeto que se puede consultar
    bajo el nombre ``pilas.mundo.control``.

    Entonces, una vez que tienes la referencia para consultar, los
    atributos que tiene este objeto control son::

        izquierda
        derecha
        arriba
        abajo
        boton

    Cada uno de estos atributos te pueden devolver True, o False, indicando
    si el control está pulsado o no.

    Ten en cuenta que este objeto también se puede imprimir usando
    la sentencia ``print``. Esto es útil para ver el estado completo
    del control de una sola vez:

        >>> print pilas.mundo.control
        <Control izquierda: False derecha: False arriba: False abajo: False boton: False>


    Consultando controles desde un actor:

    Una forma habitual de usar los controles, es consultarlos
    directamente desde el codigo de un actor.

    Para consultar los controles para cambiar la posicion horizontal de
    un actor podrías implementar el método ``actualizar``::

        class Patito(pilas.actores.Actor):

            def __init__(self):
                pilas.actores.Actor.__init__(self)
                self.imagen = "patito.png"

            def actualizar(self):
                if pilas.mundo.control.izquierda:
                    self.x -= 5
                    self.espejado = True
                elif pilas.mundo.control.derecha:
                    self.x += 5
                    self.espejado = False

    .. image:: ../../pilas/data/patito.png
    """

    def __init__(self):
        self.izquierda = False
        self.derecha = False
        self.arriba = False
        self.abajo = False
        self.boton = False

        eventos.pulsa_tecla.conectar(self.cuando_pulsa_una_tecla)
        eventos.suelta_tecla.conectar(self.cuando_suelta_una_tecla)

    def cuando_pulsa_una_tecla(self, evento):
        self.procesar_cambio_de_estado_en_la_tecla(evento.codigo, True)

    def cuando_suelta_una_tecla(self, evento):
        self.procesar_cambio_de_estado_en_la_tecla(evento.codigo, False)

    def procesar_cambio_de_estado_en_la_tecla(self, codigo, estado):
        mapa = {
            IZQUIERDA: 'izquierda',
            DERECHA: 'derecha',
            ARRIBA: 'arriba',
            ABAJO: 'abajo',
            SELECCION: 'boton',
        }

        if mapa.has_key(codigo):
            setattr(self, mapa[codigo], estado)

    def __str__(self):
        return "<Control izquierda: %s derecha: %s arriba: %s abajo: %s boton: %s>" %(
                str(self.izquierda), str(self.derecha), str(self.arriba), 
                str(self.abajo), str(self.boton))



# -*- encoding: utf-8 -*-
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")
import pilas


pilas.iniciar()

actor = pilas.actores.Actor()
imagen = pilas.imagenes.cargar_grilla("pingu.png", 10)
actor.imagen = imagen

def avanzar_cuadro(*k, **kv):
    "Avanza un cuadro de animación."
    imagen.avanzar()
    actor.imagen = imagen

pilas.eventos.click_de_mouse.connect(avanzar_cuadro)
pilas.avisar("Pulse el boton del mouse para avanzar un cuadro.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Actor


class Tortuga(Actor):
    "Representa una tortuga que se mueve por la pantalla como la tortuga de Logo."

    def __init__(self, x=0, y=0, dibuja=True):
        self.pizarra = pilas.actores.Pizarra()

        imagen = pilas.imagenes.cargar('tortuga.png')
        Actor.__init__(self, imagen, x=x, y=y)

        self.rotacion = 0
        self.velocidad = 6

        self.anterior_x = x
        self.anterior_y = y

        if dibuja:
            self.bajalapiz()
        else:
            self.subelapiz()

        self.color = pilas.colores.negro

    def avanzar(self, pasos):
        self.hacer_luego(pilas.comportamientos.Avanzar(pasos, self.velocidad))

    def giraderecha(self, delta):
        self.hacer_luego(pilas.comportamientos.Girar(abs(delta), self.velocidad))
    
    def giraizquierda(self, delta):
        self.hacer_luego(pilas.comportamientos.Girar(-abs(delta), self.velocidad))

    def actualizar(self):
        if self.anterior_x != self.x or self.anterior_y != self.y:
            self.dibujar_linea_desde_el_punto_anterior()
            self.anterior_x = self.x
            self.anterior_y = self.y

    def dibujar_linea_desde_el_punto_anterior(self):
        self.pizarra.linea(self.anterior_x, self.anterior_y, self.x, self.y, self.color, grosor=4)

    def bajalapiz(self):
        self.lapiz_bajo = True

    def subelapiz(self):
        self.lapiz_bajo = False

    def pon_color(self, color):
        self.color = color

    def crear_poligono(self, lados = 4, escala = 100, sentido = -1):
        "dibuja un poligono de n lados"
        for i in range(lados):
            rotacion = 360 / lados
            self.avanzar(escala)
            if sentido == 1:
                self.giraderecha(rotacion)
            else:
                self.giraizquierda(rotacion)
                
    def crear_circulo(self, radio = 30, sentido = -1):
        "dibuja un circulo"
        for i in range(36):
            self.avanzar(radio)
            if sentido == 1:
                self.giraderecha(10)
            else:
                self.giraizquierda(10)

    # Alias de metodos
    av = avanzar
    gd = giraderecha
    gi = giraizquierda
    bl = bajalapiz
    sl = subelapiz
    pc = pon_color


    def get_color(self):
        return self._color

    def set_color(self, color):
        self._color = color

    color = property(get_color, set_color)

    def pintar(self, color=None):
        self.pizarra.pintar(color)



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pilas

class Boton(pilas.actores.Actor):
    
    def __init__(self, texto="Sin texto", x=0, y=0, icono=None):
        pilas.actores.Actor.__init__(self, x=x, y=y)
        self.texto = texto
        self._crear_imagenes_de_botones()
        self.centro = ("centro", "centro")
        self.funcion = None

        if icono:
            self.icono = pilas.imagenes.cargar(icono)
        else:
            self.icono = None

        pilas.eventos.mueve_mouse.conectar(self.cuando_mueve_el_mouse)
        pilas.eventos.click_de_mouse.conectar(self.cuando_hace_click)

    def conectar(self, funcion):
        self.funcion = funcion

    def _crear_imagenes_de_botones(self):
        "Genera las 3 imagenes de los botones."
        ancho, alto = pilas.utils.obtener_area_de_texto(self.texto)
        tema = pilas.imagenes.cargar("boton/tema.png")

        self.imagen_normal = self._crear_imagen(tema, self.texto, ancho, 0)
        self.imagen_sobre = self._crear_imagen(tema, self.texto, ancho, 103)
        self.imagen_click = self._crear_imagen(tema, self.texto, ancho, 205)

        self.imagen = self.imagen_normal

    def cuando_mueve_el_mouse(self, evento):
        if self.colisiona_con_un_punto(evento.x, evento.y):
            self.imagen = self.imagen_sobre
        else:
            self.imagen = self.imagen_normal

    def cuando_hace_click(self, evento):
        if self.imagen == self.imagen_sobre:
            self.imagen = self.imagen_click

            if self.funcion:
                self.funcion()
        
    def _crear_imagen(self, tema, texto, ancho, dx):
        "Genera una imagen de superficie de boton."
        imagen = pilas.imagenes.cargar_superficie(20 + ancho, 30)
        imagen.pintar_parte_de_imagen(tema, dx, 0, 5, 25, 0, 0)

        for x in range(1, ancho + 20, 5):
            imagen.pintar_parte_de_imagen(tema, dx + 5, 0, 5, 25, x, 0)

        imagen.pintar_parte_de_imagen(tema, dx + 75, 0, 5, 25, ancho + 15, 0)
        imagen.texto(texto, 10, 17)
        return imagen



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Actor


class Mapa(Actor):
    """Representa mapas creados a partir de imagenes mas pequeñas.

    Este actor te permite crear escenarios tipo ``tiles``, una técnica
    de contrucción de escenarios muy popular en los videojuegos.
    
    Puedes crear un actor a partir de una grilla, e indicando cada
    uno los bloques o simplemente usando un programa externo llamado
    **tiled** (ver http://www.mapeditor.org).

    Por ejemplo, para crear un mapa desde un archivo del programa
    **tiled** puedes escribir:

        >>> mapa = pilas.actores.Mapa('untitled2.tmx')
    """

    def __init__(self, grilla_o_mapa=None, x=0, y=0, restitucion=0.56):
        Actor.__init__(self, 'invisible.png', x, y)
        self.restitucion = restitucion
        self.figuras = []
        self.bloques = []

        if not grilla_o_mapa:
            grilla_o_mapa = grilla = pilas.imagenes.cargar_grilla("grillas/plataformas_10_10.png", 10, 10)
            
        self.grilla_o_mapa = grilla_o_mapa

        if isinstance(grilla_o_mapa, str):
            self._cargar_mapa(grilla_o_mapa)
        else:
            self.grilla = grilla_o_mapa
            self._ancho_cuadro = grilla_o_mapa.cuadro_ancho
            self._alto_cuadro = grilla_o_mapa.cuadro_alto

    def _cargar_mapa(self, archivo):
        "Carga el escenario desde un archivo .tmz (del programa tiled)."

        archivo = pilas.utils.obtener_ruta_al_recurso(archivo)

        # Carga los nodos principales.
        nodo = pilas.utils.xmlreader.makeRootNode(archivo)
        nodo_mapa = nodo.getChild('map')
        nodo_tileset = nodo_mapa.getChild('tileset')

        # Cantidad de bloques en el mapa.
        self.columnas = int(nodo_mapa.getAttributeValue('width'))
        self.filas = int(nodo_mapa.getAttributeValue('height'))

        # Atributos de la imagen asociada al mapa.
        self._ruta = nodo_tileset.getChild('image').getAttributeValue('source')
        self._ruta = pilas.utils.obtener_ruta_al_recurso(self._ruta)

        self._ancho_imagen = int(nodo_tileset.getChild('image').getAttributeValue('width'))
        self._alto_imagen = int(nodo_tileset.getChild('image').getAttributeValue('height'))
        self._ancho_cuadro = int(nodo_tileset.getAttributeValue('tilewidth'))
        self._alto_cuadro = int(nodo_tileset.getAttributeValue('tileheight'))

        # Carga la grilla de imagenes desde el mapa.
        self.grilla = pilas.imagenes.cargar_grilla(self._ruta, 
                self._ancho_imagen / self._ancho_cuadro, 
                self._alto_imagen / self._alto_cuadro)

        # Carga las capas del mapa.
        layers = nodo.getChild('map').getChildren('layer')

        if len(layers) == 0:
            raise Exception("Debe tener al menos una capa (layer).")

        # La capa 0 (inferior) define los bloques no-solidos.
        self._crear_bloques(layers[0], solidos=False)

        # El resto de las capas definen bloques solidos
        for layer in layers[1:]:
            self._crear_bloques(layer, solidos=True)

    def _crear_bloques(self, capa, solidos):
        "Genera actores que representan los bloques del escenario."
        datos = capa.getChild('data').getData()

        # Convierte todo el mapa en una matriz de numeros.
        bloques = [[int(x) for x in x.split(',') if x] for x in datos.split()]

        for (y, fila) in enumerate(bloques):
            for (x, bloque) in enumerate(fila):
                if bloque:
                    self.pintar_bloque(y, x, bloque -1, solidos)

    def pintar_bloque(self, fila, columna, indice, es_bloque_solido=False):
        nuevo_bloque = pilas.actores.Actor('invisible.png')
        nuevo_bloque.imagen = self.grilla
        nuevo_bloque.imagen.definir_cuadro(indice)
        nuevo_bloque.izquierda = columna * self._ancho_cuadro - 320
        nuevo_bloque.arriba = -fila * self._alto_cuadro + 240
        self.bloques.append(nuevo_bloque)

        if es_bloque_solido:
            figura = pilas.fisica.Rectangulo(nuevo_bloque.izquierda + self._ancho_cuadro / 2, 
                    nuevo_bloque.arriba - self._alto_cuadro / 2,
                    self._ancho_cuadro, self._alto_cuadro, dinamica=False, 
                    restitucion=self.restitucion)
            self.figuras.append(figura)


    def reiniciar(self):
        self._eliminar_bloques()

        if isinstance(self.grilla_o_mapa, str):
            self._cargar_mapa(self.grilla_o_mapa)

    def eliminar(self):
        self._eliminar_bloques()

    def _eliminar_bloques(self):
        for b in self.bloques:
            b.eliminar()

        for f in self.figuras:
            f.eliminar()



# -*- encoding: utf-8 -*-
'''

import pilas
try:
    import opencv
    from opencv import highgui
except ImportError:
    opencv = None

import os

try:
    from PySFML import sf
except ImportError:
    pass

class MissingOpencv(Exception):
    def __init__(self):
        self.value = "Open CV no esta instalado, obtengalo en http://opencv.willowgarage.com"

    def __str__(self):
        return repr(self.value)

def error(biblioteca, web):
    print "Error, no ecuentra la biblioteca '%s' (de %s)" %(biblioteca, web)

def no_opencv():
    from pilas.utils import esta_en_sesion_interactiva
    if esta_en_sesion_interactiva():
        error('opencv', 'http://opencv.willowgarage.com')
    else:
        raise MissingOpencv()

class DeCamara(pilas.actores.Actor):
    """
    Nos permite poner en pantalla el video proveniente de la camara web.
    
    """
    def __init__(self, ancho=640, alto=480):
        if opencv is None:
            no_opencv()
            return
        import webcam
        self.camara = webcam.CamaraWeb
        self.ultimo_numero_de_cuadro = 0
        pilas.actores.Actor.__init__(self, 'fondos/pasto.png')
        pilas.mundo.agregar_tarea_siempre(0.15,self.actualizar_video)

    def actualizar_video(self):
        cuadro, numero_de_cuadro = self.camara.obtener_imagen(self.ultimo_numero_de_cuadro)
        self.ultimo_numero_de_cuadro = numero_de_cuadro
        self.imagen.LoadFromPixels(640, 480, cuadro)        
        return True

class VideoDeArchivo(object):
    def __init__(self, ruta):
        if opencv is None:
            no_opencv()
            return
        if not os.path.isfile(ruta):
            raise IOError('El archiyo no existe')
        self._camara = highgui.cvCreateFileCapture(ruta)
        self.fps = highgui.cvGetCaptureProperty(self._camara, highgui.CV_CAP_PROP_FPS)
        self.altura = highgui.cvGetCaptureProperty(self._camara, highgui.CV_CAP_PROP_FRAME_HEIGHT)
        self.ancho =highgui.cvGetCaptureProperty(self._camara, highgui.CV_CAP_PROP_FRAME_WIDTH)
        super(VideoDeArchivo, self).__init__()

    def obtener_imagen(self):
        imagen_ipl = highgui.cvQueryFrame(self._camara)
        imagen_ipl = opencv.cvGetMat(imagen_ipl)
        return opencv.adaptors.Ipl2PIL(imagen_ipl).convert('RGBA').tostring() 
        

class DePelicula(pilas.actores.Actor):
    """
    Nos permite poner en pantalla un video desde un archivo.
    Toma como parametro la ruta del video.
    """
    def __init__(self, path, ancho=640, alto=480):
        self._camara = VideoDeArchivo(path)
        pilas.actores.Actor.__init__(self)
        self._altura_cuadro = self._camara.altura
        self._ancho_cuadro = self._camara.ancho
        subrect = self._actor.GetSubRect()
        subrect.Right = self._ancho_cuadro
        subrect.Bottom = self._altura_cuadro
        self._actor.SetSubRect(subrect)
        self.centro = ('centro', 'centro')
        pilas.mundo.agregar_tarea_siempre(1/self._camara.fps,self.actualizar_video)

    def actualizar_video(self):
        self.imagen.LoadFromPixels(self._ancho_cuadro, self._altura_cuadro, self._camara.obtener_imagen())        
        return True
'''



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

from pilas.actores import Actor
import pilas

class EntradaDeTexto(Actor):
    """Representa una caja de texto que puede servir para ingresar texto.
    
    Este actor, en la mayoria de los casos, se utiliza para solicitarle
    el nombre a un usuario. Por ejemplo, cuando completa un record
    de puntaje."""

    def __init__(self, x=0, y=0, color=pilas.colores.negro, limite=10, tamano=32, fuente='Arial', cursor_intermitente=True):
        self.cursor = "|"
        self.texto = ""
        self.limite = limite
        imagen = pilas.imagenes.cargar_superficie(640, 480)
        Actor.__init__(self, imagen)
        pilas.eventos.pulsa_tecla.conectar(self.cuando_pulsa_una_tecla)
        self._actualizar_imagen()
        
        if cursor_intermitente:
            pilas.mundo.agregar_tarea_siempre(0.25, self._actualizar_cursor)

    def _actualizar_cursor(self):
        if self.cursor == "":
            self.cursor = "|"
        else:
            self.cursor = ""

        self._actualizar_imagen()
        return True


    def cuando_pulsa_una_tecla(self, evento):
        if evento.codigo == '\x08':
            # Indica que se quiere borrar un caracter
            self.texto = self.texto[:-1]
        else:
            if len(self.texto) < self.limite:
                self.texto = self.texto + evento.texto
        
        self._actualizar_imagen()
        
    def _actualizar_imagen(self):
        self.imagen.pintar(pilas.colores.blanco)
        self.imagen.texto(self.texto + self.cursor, 100, 100)



import pilas
from pilas.escenas import Normal

pilas.iniciar()


class EscenaDeMenu(Normal):

    def __init__(self):
        Normal.__init__(self)
        pilas.fondos.Selva()

        opciones = [
		    ('Comenzar a jugar', self.comenzar),
		    ('Salir', self.salir)]

        self.menu = pilas.actores.Menu(opciones)

    def comenzar(self):
        pilas.mundo.definir_escena(EscenaDeJuego())

    def salir(self):
        import sys
        sys.exit(0)


class EscenaDeJuego(Normal):

    def __init__(self):
        Normal.__init__(self)
        pingu = pilas.actores.Pingu()
        pilas.fondos.Pasto()
        pilas.avisar("Pulsa la tecla 'q' para regresar al menu...")

        pilas.eventos.pulsa_tecla.conectar(self.cuando_pulsa_tecla)

    def cuando_pulsa_tecla(self, evento):
        if evento.texto == u'q':
            pilas.mundo.definir_escena(EscenaDeMenu())
	
pilas.mundo.definir_escena(EscenaDeMenu())
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# For pilas engine - a video game framework.
#
# copyright 2011 - Pablo Garrido
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pilas


class Ayuda(pilas.escenas.Escena):
    "Escena que entrega instrucciones de como jugar."

    def __init__(self):
        pilas.escenas.Escena.__init__(self) 
        pilas.actores.utils.eliminar_a_todos()
        pilas.fondos.Fondo('data/ayuda.png')
        pilas.eventos.pulsa_tecla_escape.conectar(self.cuando_se_presione_escape)

    def cuando_se_presione_escape(self, *k, **kv):
        "Regresa al menu principal"
        import escena_menu
        escena_menu.EscenaMenu()



import pilas
pilas.iniciar()


def cuando_selecciona(opcion):
    print "Ha seleccionado la opcion:", opcion


consulta = pilas.interfaz.ListaSeleccion(['Uno', 'Dos', 'Tres'], cuando_selecciona)

pilas.ejecutar()
    



# -*- encoding: utf-8 -*-
import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")

pilas.iniciar()
ejes = pilas.actores.Ejes()

pilas.avisar("Mostrando el actor eje. Usa F12 para ver la posicion del mouse.")
pilas.ejecutar()



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")


pilas.iniciar()

mono = pilas.actores.Mono()
mono.escala = 1
mono.rotacion = 0

def girar():
    mono.rotacion = mono.rotacion + 1
    return True

pilas.mundo.agregar_tarea_siempre(0.01, girar)

pilas.mundo.camara.x = pilas.utils.interpolar([200, 0], duracion=10)
pilas.mundo.camara.y = pilas.utils.interpolar([200, 0], duracion=10)

pilas.avisar("Moviendo la camara...")
pilas.ejecutar()



import pilas
pilas.iniciar()


s1 = pilas.interfaz.Selector("Me gusta este selector !", x=0, y=90) 
entrada = pilas.interfaz.IngresoDeTexto()
entrada.texto = "Texto inicial"

pilas.avisar("Escribe en la caja de texto o pulsa el selector.")
pilas.ejecutar()



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")

pilas.iniciar()

pilas.fondos.Volley()

pilas.avisar("Un paisaje de ejemplo.")
pilas.ejecutar()



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")


pilas.iniciar()
pizarra = pilas.actores.Pizarra()

imagen = pilas.imagenes.cargar("pelota.png")
pizarra.pintar_imagen(imagen, 0, 0)


pilas.avisar("Imprimiendo una imagen sobre la pizarra.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.estudiante import Estudiante

IZQUIERDA = ["izquierda"]
DERECHA = ["derecha"]
ARRIBA = ["arriba", "superior"]
CENTRO = ["centro", "centrado", "medio", "arriba"]
ABAJO = ["abajo", "inferior", "debajo"]


class Actor(object, Estudiante):
    """Representa un objeto visible en pantalla, algo que se ve y tiene posicion.

    .. image:: images/actores/actor.png

    Un objeto Actor se tiene que crear siempre indicando una imagen. Si no
    se especifica una imagen, se verá una pila de color gris cómo la que
    está mas arriba.

    Una forma de crear el actor con una imagen es:

        >>> protagonista = Actor("protagonista_de_frente.png")

    incluso, es equivalente hacer lo siguiente:

        >>> imagen = pilas.imagenes.cargar("protagonista_de_frente.png")
        >>> protagonista = Actor(imagen)

    Luego, una vez que ha sido ejecutada la sentencia aparecerá
    el nuevo actor para que puedas manipularlo. Por ejemplo
    alterando sus propiedades:

        >>> protagonista.x = 100
        >>> protagonista.escala = 2
        >>> protagonista.rotacion = 30

    Estas propiedades también se pueden manipular mediante
    interpolaciones. Por ejemplo, para aumentar el tamaño del
    personaje de 1 a 5 en 7 segundos:

        >>> protagonista.escala = 1
        >>> protagonista.escala = [5], 7

    Si quieres que el actor sea invisible, un truco es crearlo
    con la imagen ``invisible.png``:

        >>> invisible = pilas.actores.Actor('invisible.png')
    """

    def __init__(self, imagen="sin_imagen.png", x=0, y=0):
        if not pilas.mundo:
            mensaje = "Tiene que invocar a la funcion ``pilas.iniciar()`` para comenzar."
            print mensaje
            raise Exception(mensaje)

        Estudiante.__init__(self)
        self._actor = pilas.mundo.motor.obtener_actor(imagen, x=x, y=y)
        self.centro = ('centro', 'centro')

        self.x = x
        self.y = y
        self.transparencia = 0

        # Define el nivel de lejanía respecto del observador.
        self.z = 0
        self._espejado = False
        self.radio_de_colision = 10
        pilas.actores.utils.insertar_como_nuevo_actor(self)
        self._transparencia = 0
        self.anexados = []

    def definir_centro(self, (x, y)):
        if type(x) == str:
            if x not in IZQUIERDA + CENTRO + DERECHA:
                raise Exception("No puedes definir '%s' como eje horizontal." %(x))
            x = self._interpretar_y_convertir_posicion(x, self.obtener_ancho())
        if type(y) == str:
            if y not in ARRIBA + CENTRO + ABAJO:
                raise Exception("No puedes definir '%s' como eje vertical." %(y))
            y = self._interpretar_y_convertir_posicion(y, self.obtener_alto())            

        self._centro = (x, y)
        self._actor.definir_centro(x, y)
    
    def _interpretar_y_convertir_posicion(self, posicion, maximo_valor):
        if posicion in IZQUIERDA + ARRIBA:
            return 0
        elif posicion in CENTRO:
            return int(maximo_valor / 2.0)
        elif posicion in DERECHA + ABAJO:
            return maximo_valor
        else:
            raise Exception("El valor '%s' no corresponde a una posicion, use numeros o valores como 'izquierda', 'arriba' etc." %(posicion))

    def obtener_centro(self):
        return self._centro
    
    centro = property(obtener_centro, definir_centro, doc="""
        Cambia la posición del punto (x, y) dentro de actor.

        Inicialmente, cuando tomamos un actor y definimos sus
        atributos (x, y). Ese punto, será el que representa el centro
        del personaje.

        Eso hace que las rotaciones sean siempre sobre el centro
        del personajes, igual que los cambios de escala y la posición.

        En algunas ocasiones, queremos que el punto (x, y) sea otra
        parte del actor. Por ejemplo sus pies. En esos casos
        es útil definir el centro del actor.

        Por ejemplo, si queremos mover el centro del actor podemos
        usar sentencias cómo estas:

            >>> actor.centro = ("izquierda", "abajo")
            >>> actor.centro = ("centro", "arriba")

        Pulsa la tecla **F8** para ver el centro del los actores
        dentro de pilas. Es aconsejable pulsar la tecla **+** para
        que el punto del modo **F8** se vea bien.
        """)

    def definir_posicion(self, x, y):
        self._actor.definir_posicion(x, y)

    def obtener_posicion(self):
        return self._actor.obtener_posicion()

    def dibujar(self, aplicacion):
        self._actor.dibujar(aplicacion)

    def get_x(self):
        x, y = self.obtener_posicion()
        return x

    @pilas.utils.interpolable
    def set_x(self, x):
        self.definir_posicion(x, self.y)

    def get_z(self):
        return self._z

    @pilas.utils.interpolable
    def set_z(self, z):
        self._z = z
        pilas.actores.utils.ordenar_actores_por_valor_z()

    @pilas.utils.interpolable
    def set_y(self, y):
        self.definir_posicion(self.x, y)

    def get_y(self):
        x, y = self.obtener_posicion()
        return y

    @pilas.utils.interpolable
    def set_scale(self, s):
        if s < 0:
            return

        ultima_escala = self.obtener_escala()

        # Se hace la siguiente regla de 3 simple:
        #
        #  ultima_escala          self.radio_de_colision
        #  s                      ?

        self.definir_escala(s)
        self.radio_de_colision = (s * self.radio_de_colision) / max(ultima_escala, 0.0001)

    def get_scale(self):
        return self.obtener_escala()

    def get_rotation(self):
        return self.obtener_rotacion()

    @pilas.utils.interpolable
    def set_rotation(self, x):
        self.definir_rotacion(x)

    def get_espejado(self):
        return self._espejado

    def set_espejado(self, nuevo_valor):
        if self._espejado != nuevo_valor:
            self._espejado = nuevo_valor
            self._actor.set_espejado(nuevo_valor)

    @pilas.utils.interpolable
    def set_transparencia(self, nuevo_valor):
        self._transparencia = nuevo_valor
        self.definir_transparencia(nuevo_valor)

    def get_transparencia(self):
        return self._transparencia

    def get_imagen(self):
        return self.obtener_imagen()

    def set_imagen(self, imagen):
        if isinstance(imagen, str):
            imagen = pilas.imagenes.cargar(imagen)

        self.definir_imagen(imagen)

    def get_fijo(self):
        return self._actor.fijo

    def set_fijo(self, fijo):
        self._actor.fijo = fijo


    espejado = property(get_espejado, set_espejado, doc="Indica si se tiene que invertir horizonaltamente la imagen del actor.")
    z = property(get_z, set_z, doc="Define lejania respecto del observador.")
    x = property(get_x, set_x, doc="Define la posición horizontal.")
    y = property(get_y, set_y, doc="Define la posición vertical.")
    rotacion = property(get_rotation, set_rotation, doc="Angulo de rotación (en grados, de 0 a 360)")
    escala = property(get_scale, set_scale, doc="Escala de tamaño, 1 es normal, 2 al doble de tamaño etc...)")
    transparencia = property(get_transparencia, set_transparencia, doc="Define el nivel de transparencia, 0 indica opaco y 100 la maxima transparencia.")
    imagen = property(get_imagen, set_imagen, doc="Define la imagen a mostrar.")
    fijo = property(get_fijo, set_fijo, doc="Indica si el actor debe ser independiente a la camara.")

    def eliminar(self):
        """Elimina el actor de la lista de actores que se imprimen en pantalla."""
        self.destruir()
        self._eliminar_anexados()
    
    def destruir(self):
        """Elimina a un actor pero de manera inmediata."""
        pilas.actores.utils.eliminar_un_actor(self)
        self.eliminar_habilidades()
        self.eliminar_comportamientos()

    def actualizar(self):
        """Actualiza el estado del actor. 
        
        Este metodo se llama una vez por frame, y generalmente se suele
        usar para implementar el comportamiento del actor.

        Si estás haciendo una subclase de Actor, es aconsejable que re-definas
        este método."""

    def pre_actualizar(self):
        """Actualiza comportamiento y habilidades antes de la actualización."""
        self.actualizar_comportamientos()
        self.actualizar_habilidades()

    def __cmp__(self, otro_actor):
        """Compara dos actores para determinar cual esta mas cerca de la camara.

        Este metodo se utiliza para ordenar los actores antes de imprimirlos
        en pantalla. De modo tal que un usuario pueda seleccionar que
        actores se ven mas arriba de otros cambiando los valores de
        los atributos `z`."""

        if otro_actor.z >= self.z:
            return 1
        else:
            return -1

    def get_izquierda(self):
        return self.x - (self.centro[0] * self.escala)

    @pilas.utils.interpolable
    def set_izquierda(self, x):
        self.x = x + (self.centro[0] * self.escala)

    izquierda = property(get_izquierda, set_izquierda)
    
    def get_derecha(self):
        return self.izquierda + self.obtener_ancho()

    @pilas.utils.interpolable
    def set_derecha(self, x):
        self.set_izquierda(x - self.ancho)

    derecha = property(get_derecha, set_derecha)

    def get_abajo(self):
        return self.get_arriba() - self.alto

    @pilas.utils.interpolable
    def set_abajo(self, y):
        self.set_arriba(y + self.alto)
        
    abajo = property(get_abajo, set_abajo)

    def get_arriba(self):
        return self.y + (self.centro[1] * self.escala)

    @pilas.utils.interpolable
    def set_arriba(self, y):
        self.y = y - (self.centro[1] * self.escala)

    arriba = property(get_arriba, set_arriba)


    def colisiona_con_un_punto(self, x, y):
        """Determina si un punto colisiona con el area del actor.

        Todos los actores tienen un area rectangular, pulsa la
        tecla **F10** para ver el area de colision.
        """
        return self.izquierda <= x <= self.derecha and self.abajo <= y <= self.arriba

    def obtener_rotacion(self):
        return self._actor.obtener_rotacion()

    def definir_rotacion(self, r):
        r = r % 360
        self._actor.definir_rotacion(r)

    def definir_color(self, c):
        self._actor.definir_color(c)

    def obtener_imagen(self):
        return self._actor.obtener_imagen()

    def definir_imagen(self, imagen):
        self._actor.definir_imagen(imagen)
        self.centro = ('centro', 'centro')

    def duplicar(self, **kv):
        duplicado = self.__class__()

        for clave in kv:
            setattr(duplicado, clave, kv[clave])

        return duplicado

    def obtener_ancho(self):
        return self.imagen.ancho()

    def obtener_alto(self):
        return self.imagen.alto()

    ancho = property(obtener_ancho)
    alto = property(obtener_alto)

    def __mul__(self, cantidad):
        if type(cantidad) is not int or cantidad < 1:
            raise TypeError("Solo puede multiplicar por numeros enteros mayores a 1.")

        grupo = pilas.atajos.fabricar(self.__class__, cantidad - 1)
        grupo.append(self)
        return grupo

    def __str__(self):
        return "<%s en (%d, %d)>" %(self.__class__.__name__, self.x, self.y)

    def obtener_escala(self):
        return self._actor.obtener_escala()

    def definir_escala(self, escala):
        self._actor.definir_escala(escala)

    def definir_transparencia(self, valor):
        self._actor.definir_transparencia(valor)

    def imitar(self, otro_actor_o_figura):
        self.aprender(pilas.habilidades.Imitar, otro_actor_o_figura)

    def esta_fuera_de_la_pantalla(self):
        # TODO: detectar area de la pantalla con las funciones que exporta el motor.
        if self.derecha < -320 or self.izquierda > 320 or self.arriba < -240 or self.abajo > 240:
            return True

    def decir(self, mensaje, autoeliminar=True):
        """Emite un mensaje usando un globo similar al de los commics"""
        nuevo_actor = pilas.actores.Globo(mensaje, self.x, self.y, autoeliminar=autoeliminar)
        nuevo_actor.z = self.z - 1
        self.anexar(nuevo_actor)

    def anexar(self, otro_actor):
        self.anexados.append(otro_actor)

    def _eliminar_anexados(self):
        for x in self.anexados:
            x.eliminar()



import pilas

MENSAJE_AYUDA = """
En Asteroides, tienes que controlar una
nave usando el teclado. El objetivo
del juego es destruir todas las piedras
del espacio disparando.

Para disparar tienes que usar la barra
espaciadora y para mover la nave
puedes usar las flechas del teclado.
"""

class Ayuda(pilas.escenas.Escena):
    "Es la escena que da instrucciones de como jugar."

    def __init__(self):
        pilas.escenas.Escena.__init__(self)
        pilas.fondos.Fondo("data/ayuda.png")
        self.crear_texto_ayuda()
        pilas.eventos.pulsa_tecla_escape.conectar(self.cuando_pulsa_tecla)

    def crear_texto_ayuda(self):
        titulo = pilas.actores.Texto("Ayuda", y=200)
        texto = pilas.actores.Texto(MENSAJE_AYUDA)
        pilas.avisar("Pulsa ESC para regresar")

    def cuando_pulsa_tecla(self, *k, **kv):
        import escena_menu
        escena_menu.EscenaMenu()



import pilas

pilas.iniciar()

texto = pilas.actores.Texto("Bienvenido.")

def cuando_pulsa_el_boton():
    texto.definir_texto("Has pulsado el boton")

b = pilas.actores.Boton(y=-100)
b.conectar_presionado(cuando_pulsa_el_boton)

pilas.avisar("Pulsa el boton para que cambie el texto.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Actor

class ListaSeleccion(Actor):

    def __init__(self, opciones, funcion_a_ejecutar=None, x=0, y=0):
        Actor.__init__(self, x=x, y=y)
        self.opciones = opciones
        self.funcion_a_ejecutar = funcion_a_ejecutar
        
        ancho, alto = pilas.mundo.motor.obtener_area_de_texto("\n".join(opciones))
        self.imagen = pilas.imagenes.cargar_superficie(int(ancho + 35), int(alto + 5))

        self._pintar_opciones()
        
        pilas.eventos.mueve_mouse.conectar(self.cuando_mueve_el_mouse)
        pilas.eventos.click_de_mouse.conectar(self.cuando_hace_click_con_el_mouse)
        self.centro = ("centro", "centro")
        
    def _pintar_opciones(self, pinta_indice_opcion=None):
        self.imagen.pintar(pilas.colores.blanco)
        
        if pinta_indice_opcion != None:
            self.imagen.rectangulo(0, pinta_indice_opcion * 19, self.imagen.ancho(), 17, relleno=True, color=pilas.colores.naranja)
        
        for indice, opcion in enumerate(self.opciones):
            self.imagen.texto(opcion, 15, y=12 + indice * 20, color=pilas.colores.negro)
        
    def cuando_mueve_el_mouse(self, evento):
        if self.colisiona_con_un_punto(evento.x, evento.y):
            opcion_seleccionada = self._detectar_opcion_bajo_el_mouse(evento)
            self._pintar_opciones(opcion_seleccionada)

    def cuando_hace_click_con_el_mouse(self, evento):
        if self.colisiona_con_un_punto(evento.x, evento.y):
            opcion = self._detectar_opcion_bajo_el_mouse(evento)
            if self.funcion_a_ejecutar:
                self.funcion_a_ejecutar(self.opciones[opcion])
            else:
                print "Cuidado, no has definido funcion a ejecutar en la lista de seleccion."

    def _detectar_opcion_bajo_el_mouse(self, evento):
        opcion = int((self.arriba - evento.y ) / 20)
        if opcion in range(0, len(self.opciones)):
            return opcion



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar
 
import math
import pilas

class Comportamiento(object):
    "Representa un comportamiento (estrategia) que se puede anexar a un actor."
    
    def iniciar(self, receptor):
        "Se invoca cuando se anexa el comportamiento a un actor."
        self.receptor = receptor

    def actualizar(self):
        """Actualiza el comportamiento en un instante dado.

        Si este metodo retorna True entonces el actor dejará
        de ejecutar este comportamiento."""
        pass

    def terminar(self):
        pass


class Girar(Comportamiento):
    "Hace girar constantemente al actor respecto de su eje de forma relativa."

    def __init__(self, delta, velocidad):
        self.delta = delta

        if delta > 0:
            self.velocidad = velocidad
        else:
            self.velocidad = -velocidad


    def iniciar(self, receptor):
        "Define el angulo inicial."
        self.receptor = receptor
        self.angulo_final = (receptor.rotacion + self.delta) % 360

    def actualizar(self):
        self.receptor.rotacion += self.velocidad

        delta = abs(self.receptor.rotacion - self.angulo_final)

        if delta <= abs(self.velocidad):
            self.receptor.rotacion = self.angulo_final
            return True

class Saltar(Comportamiento):

    def __init__(self, velocidad_inicial=10):
        self.velocidad_inicial = velocidad_inicial

    def iniciar(self, receptor):
        self.receptor = receptor
        self.suelo = int(self.receptor.y)
        self.velocidad = self.velocidad_inicial

    def actualizar(self):
        self.receptor.y += self.velocidad
        self.velocidad -= 0.3

        if self.receptor.y <= self.suelo:
            self.velocidad_inicial /= 2.0
            self.velocidad = self.velocidad_inicial
            
            if self.velocidad_inicial <= 1:
                # Si toca el suelo
                self.receptor.y = self.suelo
                return True


class Avanzar(Comportamiento):
    "Desplaza al actor en la dirección y sentido indicado por una rotación."

    def __init__(self, pasos, velocidad=5):
        self.pasos = abs(pasos)
        self.velocidad = velocidad

    def iniciar(self, receptor):
        self.receptor = receptor
        rotacion_en_radianes = math.radians(-receptor.rotacion)
        self.dx = math.cos(rotacion_en_radianes)
        self.dy = math.sin(rotacion_en_radianes)

    def actualizar(self):
        salir = False

        if self.pasos - self.velocidad < 0:
            avance = self.pasos
            salir = True
        else:
            avance = self.velocidad

        self.pasos -= avance
        self.receptor.x += self.dx * avance
        self.receptor.y += self.dy * avance

        if salir:
            return True



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")


pilas.iniciar()

pizarra = pilas.actores.Pizarra()


# Selecciona solamente a los atributos que parecen colores.
colores = [color for color in dir(pilas.colores) if not '_' in color and not color == 'pilas' and not color == "Color"]

# Por cada color imprime su nombre y un rectangulo de su color.
for indice, color in enumerate(colores):
    dy = 30 + indice * 23 - 170
    pizarra.rectangulo(0, dy, 20, 20, getattr(pilas.colores, color), relleno=True)
    pizarra.rectangulo(0, dy, 20, 20, color=pilas.colores.negro)
    pizarra.texto(color, 30, dy - 15)

pilas.avisar("Muestra los colores mas usados del modulo pilas.colores")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pilas
from pilas import pilasversion
import sys

class Depurador(object):
    """Esta clase permite hacer depuraciones visuales.
    
    La depuracion visual en pilas consiste en poder mostrar informacion
    que generalmente es invisible a los jugadores. Por ejemplo, donde
    estan situados los puntos de control, los radios de colision etc.
    
    Esta clase administra varios modos depuracion, que son los
    que dibujan figuras geometricas.
    """
    
    def __init__(self, lienzo, fps):
        self.modos = []
        self.lienzo = lienzo
        ModoDepurador.grosor_de_lineas = 1
        self.fps = fps
        self.posicion_del_mouse = (0, 0)
        pilas.eventos.mueve_mouse.conectar(self.cuando_mueve_el_mouse)
        pilas.eventos.pulsa_tecla.conectar(self.cuando_pulsa_tecla)
        
    def cuando_mueve_el_mouse(self, evento):
        self.posicion_del_mouse = (evento.x, evento.y)
        return True

    def comienza_dibujado(self, motor):
        for m in self.modos:
            m.comienza_dibujado(motor, self.lienzo)

    def dibuja_al_actor(self, motor, actor):
        for m in self.modos:
            m.dibuja_al_actor(motor, self.lienzo, actor)

    def termina_dibujado(self, motor):
        if self.modos:
            self._mostrar_cantidad_de_actores(motor)
            self._mostrar_cuadros_por_segundo(motor)
            self._mostrar_posicion_del_mouse(motor)
            self._mostrar_nombres_de_modos(motor)

            for m in self.modos:
                m.termina_dibujado(motor, self.lienzo)
    
    def cuando_pulsa_tecla(self, evento):
        if evento.codigo == 'F7':
            self._alternar_modo(ModoInformacionDeSistema)
        elif evento.codigo == 'F8':
            self._alternar_modo(ModoPuntosDeControl)
        elif evento.codigo == 'F9':
            self._alternar_modo(ModoRadiosDeColision)
        elif evento.codigo == 'F10':
            self._alternar_modo(ModoArea)
        elif evento.codigo == 'F11':
            self._alternar_modo(ModoFisica)
        elif evento.codigo == 'F12':
            self._alternar_modo(ModoPosicion)
        elif evento.texto == '+':
            self._cambiar_grosor_de_bordes(+1)
        elif evento.texto == '-':
            self._cambiar_grosor_de_bordes(-1)

    def _cambiar_grosor_de_bordes(self, cambio):
        ModoDepurador.grosor_de_lineas = max(1, ModoDepurador.grosor_de_lineas + cambio)

    def _alternar_modo(self, clase_del_modo):
        clases_activas = [x.__class__ for x in self.modos]
        
        if clase_del_modo in clases_activas:
            self._desactivar_modo(clase_del_modo)
        else:
            self._activar_modo(clase_del_modo)
    
    def _activar_modo(self, clase_del_modo):
        pilas.eventos.inicia_modo_depuracion.send('depurador')
        instancia_del_modo = clase_del_modo(self)
        self.modos.append(instancia_del_modo)
        # Ordena todos los registros por numero de tecla.
        self.modos.sort(key=lambda x: x.orden_de_tecla())
        
    def _desactivar_modo(self, clase_del_modo):
        instancia_a_eliminar = [x for x in self.modos 
                                if x.__class__ == clase_del_modo]
        self.modos.remove(instancia_a_eliminar[0])
        
        if not self.modos:
            pilas.eventos.sale_modo_depuracion.send('depurador')

    def _mostrar_nombres_de_modos(self, motor):
        dy = 0
        izquierda, derecha, arriba, abajo = pilas.utils.obtener_bordes()

        for modo in self.modos:
            texto = modo.tecla + " " + modo.__class__.__name__ + " habilitado."
            self.lienzo.texto_absoluto(motor, texto, izquierda + 10, arriba -20 +dy, 
                    color=pilas.colores.violeta)
            dy -= 20
            
    def _mostrar_posicion_del_mouse(self, motor):
        izquierda, derecha, arriba, abajo = pilas.utils.obtener_bordes()
        x, y = self.posicion_del_mouse    
        texto = u"Posición del mouse: x=%d y=%d " %(x, y)
        self.lienzo.texto_absoluto(motor, texto, derecha - 230, abajo + 10, color=pilas.colores.violeta)
        
    def _mostrar_cuadros_por_segundo(self, motor):
        izquierda, derecha, arriba, abajo = pilas.utils.obtener_bordes()
        rendimiento = self.fps.obtener_cuadros_por_segundo()
        texto = "Cuadros por segundo: %s" %(rendimiento)
        self.lienzo.texto_absoluto(motor, texto, izquierda + 10, abajo + 10, 
                color=pilas.colores.violeta)

    def _mostrar_cantidad_de_actores(self, motor):
        izquierda, derecha, arriba, abajo = pilas.utils.obtener_bordes()
        total_de_actores = len(pilas.actores.todos)
        texto = "Cantidad de actores: %s" %(total_de_actores)
        self.lienzo.texto_absoluto(motor, texto, izquierda + 10, abajo + 30, 
                color=pilas.colores.violeta)
        
class ModoDepurador(object):
    tecla = "F00"

    def __init__(self, depurador):
        self.depurador = depurador
        
    def comienza_dibujado(self, motor, lienzo):
        pass
    
    def dibuja_al_actor(self, motor, lienzo, actor):
        pass

    def termina_dibujado(self, motor, lienzo):
        pass
     
    def orden_de_tecla(self):
        return int(self.tecla[1:])
    
class ModoPuntosDeControl(ModoDepurador):
    tecla = "F8"
    
    def dibuja_al_actor(self, motor, lienzo, actor):
        lienzo.cruz(motor, actor.x, actor.y, color=pilas.colores.rojo, grosor=ModoDepurador.grosor_de_lineas)
        
class ModoRadiosDeColision(ModoDepurador):
    tecla = "F9"
    
    def dibuja_al_actor(self, motor, lienzo, actor):
        lienzo.circulo(motor, actor.x, actor.y, actor.radio_de_colision, color=pilas.colores.verde, grosor=ModoDepurador.grosor_de_lineas)
 
class ModoArea(ModoDepurador):
    tecla = "F10"
    
    def dibuja_al_actor(self, motor, lienzo, actor):
        dx, dy = actor.centro
        lienzo.rectangulo(motor, actor.x - dx, actor.y + dy, actor.ancho, actor.alto, color=pilas.colores.azul, grosor=ModoDepurador.grosor_de_lineas)

class ModoPosicion(ModoDepurador):
    tecla = "F12"
    
    def __init__(self, depurador):
        ModoDepurador.__init__(self, depurador)

    def dibuja_al_actor(self, motor, lienzo, actor):
        if not isinstance(actor, pilas.fondos.Fondo):
            texto = "(%d, %d)" %(actor.x, actor.y)
            lienzo.texto(motor, texto, actor.derecha, actor.abajo, color=pilas.colores.violeta)
        
class ModoFisica(ModoDepurador):
    tecla = "F11"
    
    def termina_dibujado(self, motor, lienzo):
        grosor = ModoDepurador.grosor_de_lineas
        pilas.mundo.fisica.dibujar_figuras_sobre_lienzo(motor, lienzo, grosor)

class ModoInformacionDeSistema(ModoDepurador):
    tecla = "F7"

    def __init__(self, depurador):
        ModoDepurador.__init__(self, depurador)

        self.informacion = [
            "Usando el motor: " + pilas.mundo.motor.__class__.__name__,
            "Sistema: " + sys.platform,
            "Version de pilas: " + pilasversion.VERSION,
            "Version de python: " + sys.subversion[0] + " " + sys.subversion[1],
            ]
    
    def termina_dibujado(self, motor, lienzo):
        izquierda, derecha, arriba, abajo = pilas.utils.obtener_bordes()

        for (i, texto) in enumerate(self.informacion):
            posicion_y = abajo + 50 + i * 20
            lienzo.texto(motor, texto, izquierda + 10, posicion_y, color=pilas.colores.negro)



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")

pilas.iniciar()
mono = pilas.actores.Mono(y=-100)

def cuando_cambia_escala(valor):
    mono.escala = valor * 2

deslizador_escala = pilas.interfaz.Deslizador(y=50)
deslizador_escala.conectar(cuando_cambia_escala)


def cuando_cambia_rotacion(valor):
    mono.rotacion = valor * 360

deslizador_rotacion = pilas.interfaz.Deslizador(y=100)
deslizador_rotacion.conectar(cuando_cambia_rotacion)


def cuando_cambia_posicion(valor):
    # Obtiene valores entre -200 y 400
    mono.x = -200 + 400 * valor
    print valor


deslizador_posicion = pilas.interfaz.Deslizador(y=150)
deslizador_posicion.conectar(cuando_cambia_posicion)

pilas.avisar("Usa el deslizador para modificar al mono.")
pilas.ejecutar()



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")


pilas.iniciar()
mono = pilas.actores.Mono()

pasos = 3

mono.x = -50
mono.y = -50

mono.hacer_luego(pilas.comportamientos.Avanzar(90, pasos))
mono.hacer_luego(pilas.comportamientos.Girar(180, 10))
mono.hacer_luego(pilas.comportamientos.Avanzar(180, pasos))
mono.hacer_luego(pilas.comportamientos.Girar(-180, 10))
mono.hacer_luego(pilas.comportamientos.Avanzar(180, pasos))

pilas.avisar("Movimiento mediante comportamientos.")
pilas.ejecutar()



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")

pilas.iniciar()

class DibujoDeLineas:

    def __init__(self):
        self.mouse_x = 0
        self.mouse_y = 0
        self.boton_pulsado = False
        self.pizarra = pilas.actores.Pizarra()
        pilas.eventos.click_de_mouse.conectar(self.cuando_pulsa_el_boton)
        pilas.eventos.mueve_mouse.conectar(self.cuando_mueve_el_mouse)
        pilas.eventos.termina_click.conectar(self.cuando_deja_de_pulsar_el_boton)

    def cuando_pulsa_el_boton(self, evento):
        self.boton_pulsado = True

    def cuando_deja_de_pulsar_el_boton(self, evento):
        self.boton_pulsado = False

    def cuando_mueve_el_mouse(self, evento):
        if self.boton_pulsado:
            self.pizarra.linea(self.mouse_x, self.mouse_y, evento.x, evento.y, grosor=2)

        self.mouse_x = evento.x
        self.mouse_y = evento.y
    
a = DibujoDeLineas()

pilas.avisar("Use el mouse para dibujar.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pilas
from pilas import colores

try:
    import Box2D as box2d
except ImportError:
    print "No esta disponible box2d, se deshabilitara la fisica."

import math

class Fisica(object):
    "Representa un simulador de mundo fisico, usando la biblioteca box2d."
    
    def __init__(self, area, gravedad=(0, -90)):
        self.area = area
        try:
            self.escenario = box2d.b2AABB()
            self.escenario.lowerBound = (-1000.0, -1000.0)
            self.escenario.upperBound = (1000.0, 1000.0)
            self.gravedad = box2d.b2Vec2(gravedad[0], gravedad[1])
            try:
                self.mundo = box2d.b2World(self.escenario, self.gravedad, True)
            except ValueError:
                print "Solo esta disponible el motor de fisica para box2d 2.0.2b1"
                raise AttributeError("...")
        except AttributeError:
            print "Deshabilitando modulo de fisica (no se encuentra instalado pybox2d en este equipo)"
            self.mundo = None
            return

        self.constante_mouse = None
        self.i = 0
        self.crear_bordes_del_escenario()
        self.figuras_a_eliminar = []

    def crear_bordes_del_escenario(self):
        self.crear_techo(self.area)
        self.crear_suelo(self.area)
        self.crear_paredes(self.area)

    def reiniciar(self):
        for x in self.mundo.bodyList:
            self.mundo.DestroyBody(x)

        self.crear_bordes_del_escenario()

    def capturar_figura_con_el_mouse(self, figura):
        if self.constante_mouse:
            self.cuando_suelta_el_mouse()
        
        self.constante_mouse = ConstanteDeMovimiento(figura)
    
    def cuando_mueve_el_mouse(self, x, y):
        if self.constante_mouse:
            self.constante_mouse.mover(x, y)

    def cuando_suelta_el_mouse(self):
        if self.constante_mouse:
            self.constante_mouse.eliminar()
            self.constante_mouse = None
        
    def actualizar(self):
        if self.mundo:
            self.mundo.Step(1.0 / 20.0, 10, 8)
            self.i += 1
            self._procesar_figuras_a_eliminar()

    def _procesar_figuras_a_eliminar(self):
        "Elimina las figuras que han sido marcadas para quitar."
        if self.figuras_a_eliminar:
            for x in self.figuras_a_eliminar:
                # Solo elimina las figuras que actualmente existen.
                if x in self.mundo.bodyList:
                    self.mundo.DestroyBody(x)
            self.figuras_a_eliminar = []

    def dibujar_figuras_sobre_lienzo(self, motor, lienzo, grosor=1):
        "Dibuja todas las figuras en una pizarra. Indicado para depuracion."
        cuerpos = self.mundo.bodyList
        cantidad_de_figuras = 0
        
        for cuerpo in cuerpos:
            xform = cuerpo.GetXForm()
            
            for figura in cuerpo.shapeList:
                cantidad_de_figuras += 1
                tipo_de_figura = figura.GetType()
                
                if tipo_de_figura == box2d.e_polygonShape:
                    vertices = []
                    
                    for v in figura.vertices:
                        pt = box2d.b2Mul(xform, v)
                        
                        vertices.append((pt.x, pt.y))
                        
                    lienzo.poligono(motor, vertices, color=colores.rojo, grosor=grosor, cerrado=True)
                    
                elif tipo_de_figura == box2d.e_circleShape:
                    lienzo.circulo(motor, cuerpo.position.x, cuerpo.position.y, figura.radius, colores.rojo, grosor=grosor)
                else:
                    print "no puedo identificar el tipo de figura."
        
    def crear_cuerpo(self, definicion_de_cuerpo):
        return self.mundo.CreateBody(definicion_de_cuerpo)
    
    def crear_suelo(self, (ancho, alto), restitucion=0):
        self.suelo = Rectangulo(0, -alto/2, ancho, 2, dinamica=False, fisica=self, restitucion=restitucion)

    def crear_techo(self, (ancho, alto), restitucion=0):
        self.techo = Rectangulo(0, alto/2, ancho, 2, dinamica=False, fisica=self, restitucion=restitucion)
        
    def crear_paredes(self, (ancho, alto), restitucion=0):
        self.pared_izquierda = Rectangulo(-ancho/2, 0, 2, alto, dinamica=False, fisica=self, restitucion=restitucion)
        self.pared_derecha = Rectangulo(ancho/2, 0, 2, alto, dinamica=False, fisica=self, restitucion=restitucion)
        
    def eliminar_suelo(self):
        if self.suelo:
            self.suelo.eliminar()
            self.suelo = None

    def eliminar_techo(self):
        if self.techo:
            self.techo.eliminar()
            self.techo = None
            
    def eliminar_paredes(self):
        if self.pared_izquierda:
            self.pared_derecha.eliminar()
            self.pared_izquierda.eliminar()
            self.pared_derecha = None
            self.pared_izquierda = None
    
    def eliminar_figura(self, figura):
        self.figuras_a_eliminar.append(figura)
        
    def obtener_distancia_al_suelo(self, x, y, dy):
        """Obtiene la distancia hacia abajo desde el punto (x,y). 
        
        El valor de 'dy' tiene que ser positivo.
        
        Si la funcion no encuentra obstaculos retornara
        dy, pero en paso contrario retornara un valor menor
        a dy.
        """
        
        if dy < 0:
            raise Exception("El valor de 'dy' debe ser positivo, ahora vale '%f'." %(dy))

        delta = 0
        
        while delta < dy:
            
            if self.obtener_cuerpos_en(x, y-delta):
                return delta
            
            delta += 1
            
        return delta

    def obtener_cuerpos_en(self, x, y):
        "Retorna una lista de cuerpos que se encuentran en la posicion (x, y) o retorna una lista vacia []."

        AABB = box2d.b2AABB()
        f = 1
        AABB.lowerBound = (x-f, y-f)
        AABB.upperBound = (x+f, y+f)

        cuantos, cuerpos = self.mundo.Query(AABB, 2)

        if cuantos == 0:
            return []
        
        lista_de_cuerpos = []
        
        for s in cuerpos:
            cuerpo = s.GetBody()
                    
            if s.TestPoint(cuerpo.GetXForm(), (x, y)):
                lista_de_cuerpos.append(cuerpo)

        return lista_de_cuerpos
    
    def definir_gravedad(self, x, y):
        pilas.fisica.definir_gravedad(x, y)

class Figura(object):
    "Representa un figura que simula un cuerpo fisico."

    def obtener_x(self):
        return self._cuerpo.position.x

    def definir_x(self, x):
        self._cuerpo.SetXForm((x, self.y), self._cuerpo.GetAngle())

    def obtener_y(self):
        return self._cuerpo.position.y

    def definir_y(self, y):
        self._cuerpo.SetXForm((self.x, y), self._cuerpo.GetAngle())

    def obtener_rotacion(self):
        return - math.degrees(self._cuerpo.GetAngle())

    def definir_rotacion(self, angulo):
        self._cuerpo.SetXForm((self.x, self.y), math.radians(-angulo))
        
    def impulsar(self, dx, dy):
        self._cuerpo.ApplyImpulse((dx, dy), self._cuerpo.GetWorldCenter())
        
    def obtener_velocidad_lineal(self):
        velocidad = self._cuerpo.GetLinearVelocity()
        return (velocidad.x, velocidad.y)

    def detener(self):
        """Hace que la figura regrese al reposo."""
        self.definir_velocidad_lineal(0, 0)

    def definir_velocidad_lineal(self, dx=None, dy=None):
        anterior_dx, anterior_dy = self.obtener_velocidad_lineal()

        if dx is None:
            dx = anterior_dx
        if dy is None:
            dy = anterior_dy

        self._cuerpo.SetLinearVelocity((dx, dy))

    def empujar(self, dx=None, dy=None):
        self.definir_velocidad_lineal(dx, dy)
        
    def eliminar(self):
        """Quita una figura de la simulación."""
        pilas.mundo.fisica.eliminar_figura(self._cuerpo)
        
    x = property(obtener_x, definir_x, doc="define la posición horizontal.")
    y = property(obtener_y, definir_y, doc="define la posición vertical.")
    rotacion = property(obtener_rotacion, definir_rotacion, doc="define la rotacion.")
    
class Circulo(Figura):
    "Representa un cuerpo de circulo."
    
    def __init__(self, x, y, radio, dinamica=True, densidad=1.0, 
            restitucion=0.56, friccion=10.5, amortiguacion=0.1, 
            fisica=None):

        if not fisica:
            fisica = pilas.mundo.fisica
            
        bodyDef = box2d.b2BodyDef()
        bodyDef.position=(x, y)
        bodyDef.linearDamping = amortiguacion
        #userData = { 'color' : self.parent.get_color() }
        #bodyDef.userData = userData
        #self.parent.element_count += 1
        
        body = fisica.crear_cuerpo(bodyDef)

        # Create the Body
        if not dinamica:
            densidad = 0

        # Add a shape to the Body
        circleDef = box2d.b2CircleDef()
        circleDef.density = densidad
        circleDef.radius = radio
        circleDef.restitution = restitucion
        circleDef.friction = friccion
        
        body.CreateShape(circleDef)
        body.SetMassFromShapes()    

        self._cuerpo = body

class Rectangulo(Figura):
    def __init__(self, x, y, ancho, alto, dinamica=True, densidad=1.0, 
            restitucion=0.56, friccion=10.5, amortiguacion=0.1, 
            fisica=None):

        if not fisica:
            fisica = pilas.mundo.fisica

        bodyDef = box2d.b2BodyDef()
        bodyDef.position=(x, y)
        bodyDef.linearDamping = amortiguacion
        
        #userData = { 'color' : self.parent.get_color() }
        #bodyDef.userData = userData
        #self.parent.element_count += 1
        
        body = fisica.crear_cuerpo(bodyDef)

        # Create the Body
        if not dinamica:
            densidad = 0

        # Add a shape to the Body
        boxDef = box2d.b2PolygonDef()
        
        boxDef.SetAsBox(ancho/2, alto/2, (0,0), 0)
        boxDef.density = densidad
        boxDef.restitution = restitucion
        boxDef.friction = friccion
        body.CreateShape(boxDef)

        body.SetMassFromShapes()    

        self._cuerpo = body

class ConstanteDeMovimiento():

    def __init__(self, figura):
        md = box2d.b2MouseJointDef()
        mundo = pilas.mundo.fisica.mundo
        md.body1 = mundo.GetGroundBody()
        md.body2 = figura._cuerpo
        md.target = (figura.x, figura.y)
        md.maxForce = 5000.0 * figura._cuerpo.GetMass()

        try:
            self.constante = mundo.CreateJoint(md).getAsType()
        except:
            self.constante = mundo.CreateJoint(md)

        figura._cuerpo.WakeUp()

    def mover(self, x, y):
        self.constante.SetTarget((x, y))

    def eliminar(self):
        pilas.mundo.fisica.mundo.DestroyJoint(self.constante)

class ConstanteDeDistancia():
    """Representa una distancia fija entre dos figuras.

    Esta constante es útil para representar ejes o barras
    que sostienen dos cuerpos. Por ejemplo, un eje entre dos
    ruedas en un automóvil:

        >>> circulo_1 = pilas.fisica.Circulo(-100, 0, 50)
        >>> circulo_2 = pilas.fisica.Circulo(100, 50, 50)
        >>> barra = pilas.fisica.ConstanteDeDistancia(circulo_1, circulo_2)

    La distancia que tiene que respetarse en la misma que tienen
    las figuras en el momento en que se establece la constante.
    """

    def __init__(self, figura_1, figura_2, fisica=None):
        if not fisica:
            fisica = pilas.mundo.fisica
            
        if not isinstance(figura_1, Figura) or not isinstance(figura_2, Figura):
            raise Exception("Las dos figuras tienen que ser objetos de la clase Figura.")
        
        constante = box2d.b2DistanceJointDef()
        constante.Initialize(figura_1._cuerpo, figura_2._cuerpo, (0,0), (0,0))
        constante.collideConnected = True
        self.constante = fisica.mundo.CreateJoint(constante)


    def eliminar(self):
        pilas.mundo.fisica.mundo.DestroyJoint(self.constante_mouse)

def definir_gravedad(x=0, y=-90):
    pilas.mundo.fisica.mundo.gravity = (x, y)



import pilas

pilas.iniciar()

# Genera al personaje que habla
aceituna = pilas.actores.Aceituna(x=-100)


# Se crean los dos botones.
b1 = pilas.actores.Boton(x=100, y=50)
b2 = pilas.actores.Boton(x=100, y=-50)


# Generamos el administrador de los dialogos.
dialogo = pilas.actores.Dialogo(modo_automatico=False)


# Ahora las acciones que vamos a ejecutar cuando
# se pulsan los botones.

def cuando_pulsa_el_boton(texto):
    dialogo.decir_inmediatamente(aceituna, "Has pulsado: " + texto)


# Y conectamos las funciones con los botones.
b1.conectar_presionado(cuando_pulsa_el_boton, "boton de arriba")
b2.conectar_presionado(cuando_pulsa_el_boton, "boton de abajo")

pilas.avisar("Pulsa alguno de los botones")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2011 - Diego Sarmentero
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar
#
# module based in NINJA-IDE "locator.py" (http://ninja-ide.org)

from PyQt4.QtGui import QWidget
from PyQt4.QtGui import QListWidgetItem
from PyQt4.QtGui import QListWidget
from PyQt4.QtGui import QVBoxLayout
from PyQt4.QtGui import QFrame
from PyQt4.QtGui import QLineEdit
from PyQt4.QtGui import QHBoxLayout
from PyQt4.QtCore import SIGNAL
from PyQt4.QtCore import Qt


class ExampleLocatorWidget(QWidget):

    def __init__(self, data, parent):
        QWidget.__init__(self)
        self._parentTree = parent
        self.data = data

        hLocator = QHBoxLayout(self)
        hLocator.setContentsMargins(0, 0, 0, 0)
        self._completer = LocateCompleter(self)

        hLocator.addWidget(self._completer)

        self.connect(self._completer, SIGNAL("itemFound(QString)"),
            self.open_item)

    def setFocus(self):
        self._completer.setFocus()

    def open_item(self, path):
        self.emit(SIGNAL("itemFound(QString)"), path)


class LocateItem(QListWidgetItem):

    def __init__(self, name, path):
        QListWidgetItem.__init__(self, name)
        self.path = path


class LocateCompleter(QLineEdit):

    def __init__(self, parent=None):
        QLineEdit.__init__(self, parent)
        self._parent = parent
        self.setPlaceholderText("Buscar... (Ctrl+F)")
        self.__prefix = ''
        self.frame = None
        self.tempLocations = []

        self.connect(self, SIGNAL("textChanged(QString)"),
            self.set_prefix)

    def set_prefix(self, prefix):
        self.__prefix = unicode(prefix.toLower())
        self._refresh_filter()

    def complete(self):
        self.frame = PopupCompleter(self.filter())
        self.frame.setFixedWidth(self._parent._parentTree.width())
        point = self._parent.mapToGlobal(self.pos())
        self.frame.show()
        self.frame.move(point.x(), point.y() - self.frame.height())

    def filter(self):
        self.tempLocations = self._parent.data
        if self.__prefix:
            self.tempLocations = [LocateItem(x, self.tempLocations[x])
                for x in self.tempLocations
                if x.lower().find(self.__prefix) > -1]
        else:
            self.tempLocations = [LocateItem(x, self.tempLocations[x])
                for x in self.tempLocations]
        return self.tempLocations

    def _refresh_filter(self):
        self.frame.refresh(self.filter())

    def keyPressEvent(self, event):
        if event.key() == Qt.Key_Space:
            currentText = unicode(self.frame.listWidget.currentItem().text())
            self.setText(currentText)
            return

        QLineEdit.keyPressEvent(self, event)
        if event.key() == Qt.Key_Down:
            if self.frame.listWidget.currentRow() != \
              self.frame.listWidget.count() - 1:
                self.frame.listWidget.setCurrentRow(
                    self.frame.listWidget.currentRow() + 1)
        elif event.key() == Qt.Key_Up:
            if self.frame.listWidget.currentRow() > 0:
                self.frame.listWidget.setCurrentRow(
                    self.frame.listWidget.currentRow() - 1)
        elif event.key() in (Qt.Key_Tab, Qt.Key_Return, Qt.Key_Enter):
            item = self.frame.listWidget.currentItem()
            self.emit(SIGNAL("itemFound(QString)"), item.path)
            self.setText('')
            self.frame.hide()
            self._parent._parentTree.setFocus()
        elif event.key() == Qt.Key_Escape:
            self.frame.hide()
            self._parent._parentTree.setFocus()

    def focusOutEvent(self, event):
        if self.frame:
            self.frame.hide()
        QLineEdit.focusOutEvent(self, event)

    def focusInEvent(self, event):
        QLineEdit.focusOutEvent(self, event)
        self.complete()


class PopupCompleter(QFrame):

    def __init__(self, model):
        QFrame.__init__(self, None, Qt.FramelessWindowHint | Qt.ToolTip)
        vbox = QVBoxLayout(self)
        vbox.setContentsMargins(0, 0, 0, 0)
        vbox.setSpacing(0)
        self.listWidget = QListWidget()
        for i, item in enumerate(model):
            self.listWidget.addItem(item)
        self.listWidget.setCurrentRow(0)
        vbox.addWidget(self.listWidget)

    def refresh(self, model):
        self.listWidget.clear()
        for i, item in enumerate(model):
            self.listWidget.addItem(item)
        self.listWidget.setCurrentRow(0)



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")

pilas.iniciar()

class Patito(pilas.actores.Actor):

    def __init__(self):
        pilas.actores.Actor.__init__(self)
        self.imagen = "patito.png"

    def actualizar(self):
        if pilas.mundo.control.izquierda:
            self.x -= 5
            self.espejado = True
        elif pilas.mundo.control.derecha:
            self.x += 5
            self.espejado = False


pilas.fondos.Noche()
patito = Patito()
pilas.avisar("Usa el teclado para mover al patito.")
pilas.ejecutar()



import pilas
# Permite que este ejemplo funcion incluso si no has instalado pilas.
import sys
sys.path.insert(0, "..")


pilas.iniciar()


puntaje = pilas.actores.Puntaje()


texto = pilas.actores.Texto('Puntaje:')
texto.x = -80
texto.y = -2

def sumar_5_al_clickear(evento):
    puntaje.aumentar(5)
    
pilas.eventos.click_de_mouse.conectar(sumar_5_al_clickear)
pilas.avisar('Clickea la pantalla y agregaras 5 puntos al puntaje')

pilas.ejecutar()



# -*- encoding: utf-8 -*-
# For pilas engine - a video game framework.
#
# copyright 2011 - Pablo Garrido
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pilas

pilas.iniciar(titulo = "Memorice")




# ejecuta escena actual.
import escena_menu
escena_menu.EscenaMenu()



pilas.ejecutar()



'''
import pilas
try:
    import Image
    import opencv
    from opencv import highgui 

    GLOBALCAM=highgui.cvCreateCameraCapture(0)

    for algo in range(30):
        ULTIMO_CUADRO_BASURA = highgui.cvQueryFrame(GLOBALCAM)

    ULTIMO_CUADRO_BASURA = opencv.adaptors.Ipl2PIL(opencv.cvGetMat(ULTIMO_CUADRO_BASURA)).convert('RGBA')
except ImportError:
    print "Falta la biblioteca opencv o PIL"
    pass


class __camara_buffer(object):
    def __init__(self):
        self._ultimo_numero_de_cuadro = 0
        self._camera = GLOBALCAM
        self._ultimo_cuadro = ULTIMO_CUADRO_BASURA.tostring() 
        
    def _obtener_imagen_de_camara(self):
        imagen_ipl = highgui.cvQueryFrame(self._camera)
        imagen_ipl = opencv.cvGetMat(imagen_ipl)
        self._ultimo_cuadro = opencv.adaptors.Ipl2PIL(imagen_ipl).convert('RGBA').tostring() 

    def obtener_imagen(self, numero_de_cuadro=0):
        if numero_de_cuadro == self._ultimo_numero_de_cuadro:
           self._obtener_imagen_de_camara()
           self._ultimo_numero_de_cuadro += 1
        return self._ultimo_cuadro, self._ultimo_numero_de_cuadro

CamaraWeb = __camara_buffer()
'''
